@@ -10,8 +10,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import warnings
+
 from openstack.block_storage import _base_proxy
 from openstack.block_storage.v2 import backup as _backup
+from openstack.block_storage.v2 import capabilities as _capabilities
+from openstack.block_storage.v2 import extension as _extension
+from openstack.block_storage.v2 import limits as _limits
+from openstack.block_storage.v2 import quota_class_set as _quota_class_set
 from openstack.block_storage.v2 import quota_set as _quota_set
 from openstack.block_storage.v2 import snapshot as _snapshot
 from openstack.block_storage.v2 import stats as _stats
@@ -19,6 +25,7 @@ from openstack.block_storage.v2 import type as _type
 from openstack.block_storage.v2 import volume as _volume
 from openstack.identity.v3 import project as _project
 from openstack import resource
+from openstack import warnings as os_warnings
 
 
 class Proxy(_base_proxy.BaseBlockStorageProxy):
@@ -31,7 +38,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             instance.
 
         :returns: One :class:`~openstack.block_storage.v2.snapshot.Snapshot`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_snapshot.Snapshot, snapshot)
@@ -48,7 +55,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         :param snapshot: The name or ID a snapshot
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the snapshot does not exist. When set to ``True``, None will
             be returned when attempting to find a nonexistent resource.
         :param bool details: When set to ``False``, an
@@ -63,7 +70,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: One :class:`~openstack.block_storage.v2.snapshot.Snapshot`,
             one :class:`~openstack.block_storage.v2.snapshot.SnapshotDetail`
             object, or None.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         :raises: :class:`~openstack.exceptions.DuplicateResource` when multiple
             resources are found.
@@ -124,7 +131,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             :class:`~openstack.block_storage.v2.snapshot.Snapshot`
             instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the snapshot does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent snapshot.
@@ -156,11 +163,31 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             :class:`~openstack.block_storage.v2.type.Type` instance.
 
         :returns: One :class:`~openstack.block_storage.v2.type.Type`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_type.Type, type)
 
+    def find_type(self, name_or_id, ignore_missing=True):
+        """Find a single volume type
+
+        :param snapshot: The name or ID a volume type
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            when the volume type does not exist.
+
+        :returns: One :class:`~openstack.block_storage.v2.type.Type`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+            when no resource can be found.
+        :raises: :class:`~openstack.exceptions.DuplicateResource` when multiple
+            resources are found.
+        """
+        return self._find(
+            _type.Type,
+            name_or_id,
+            ignore_missing=ignore_missing,
+        )
+
     def types(self, **query):
         """Retrieve a generator of volume types
 
@@ -186,7 +213,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param type: The value can be either the ID of a type or a
             :class:`~openstack.block_storage.v2.type.Type` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the type does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent type.
@@ -241,7 +268,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             :class:`~openstack.block_storage.v2.volume.Volume` instance.
 
         :returns: One :class:`~openstack.block_storage.v2.volume.Volume`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_volume.Volume, volume)
@@ -258,7 +285,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         :param volume: The name or ID a volume
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the volume does not exist.
         :param bool details: When set to ``False`` no extended attributes
             will be returned. The default, ``True``, will cause an object with
@@ -269,7 +296,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         :returns: One :class:`~openstack.block_storage.v2.volume.Volume` or
             None.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         :raises: :class:`~openstack.exceptions.DuplicateResource` when multiple
             resources are found.
@@ -326,7 +353,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param volume: The value can be either the ID of a volume or a
             :class:`~openstack.block_storage.v2.volume.Volume` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the volume does not exist.  When set to ``True``, no
             exception will be set when attempting to delete a nonexistent
             volume.
@@ -353,12 +380,27 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume = self._get_resource(_volume.Volume, volume)
         volume.extend(self, size)
 
+    def set_volume_readonly(self, volume, readonly=True):
+        """Set a volume's read-only flag.
+
+        :param volume: The value can be either the ID of a volume or a
+            :class:`~openstack.block_storage.v2.volume.Volume` instance.
+        :param bool readonly: Whether the volume should be a read-only volume
+            or not.
+
+        :returns: None
+        """
+        volume = self._get_resource(_volume.Volume, volume)
+        volume.set_readonly(self, readonly)
+
     def retype_volume(self, volume, new_type, migration_policy="never"):
         """Retype the volume.
 
         :param volume: The value can be either the ID of a volume or a
             :class:`~openstack.block_storage.v2.volume.Volume` instance.
-        :param str new_type: The new volume type that volume is changed with.
+        :param new_type: The new volume type that volume is changed with.
+            The value can be either the ID of the volume type or a
+            :class:`~openstack.block_storage.v2.type.Type` instance.
         :param str migration_policy: Specify if the volume should be migrated
             when it is re-typed. Possible values are on-demand or never.
             Default: never.
@@ -366,7 +408,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: None
         """
         volume = self._get_resource(_volume.Volume, volume)
-        volume.retype(self, new_type, migration_policy)
+        type_id = resource.Resource._get_id(new_type)
+        volume.retype(self, type_id, migration_policy)
 
     def set_volume_bootable_status(self, volume, bootable):
         """Set bootable status of the volume.
@@ -381,8 +424,38 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume = self._get_resource(_volume.Volume, volume)
         volume.set_bootable_status(self, bootable)
 
+    def set_volume_image_metadata(self, volume, **metadata):
+        """Update image metadata for a volume
+
+        :param volume: Either the ID of a volume or a
+            :class:`~openstack.block_storage.v2.volume.Volume`.
+        :param kwargs metadata: Key/value pairs to be updated in the volume's
+            image metadata. No other metadata is modified by this call.
+
+        :returns: None
+        """
+        volume = self._get_resource(_volume.Volume, volume)
+        return volume.set_image_metadata(self, metadata=metadata)
+
+    def delete_volume_image_metadata(self, volume, keys=None):
+        """Delete metadata for a volume
+
+        :param volume: Either the ID of a volume or a
+            :class:`~openstack.block_storage.v2.volume.Volume`.
+        :param list keys: The keys to delete. If left empty complete
+            metadata will be removed.
+
+        :returns: None
+        """
+        volume = self._get_resource(_volume.Volume, volume)
+        if keys is not None:
+            for key in keys:
+                volume.delete_image_metadata_item(self, key)
+        else:
+            volume.delete_image_metadata(self)
+
     def reset_volume_status(
-        self, volume, status, attach_status, migration_status
+        self, volume, status=None, attach_status=None, migration_status=None
     ):
         """Reset volume statuses.
 
@@ -476,7 +549,6 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume.complete_migration(self, new_volume, error)
 
     # ====== BACKEND POOLS ======
-
     def backend_pools(self, **query):
         """Returns a generator of cinder Back-end storage pools
 
@@ -530,14 +602,14 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
 
         :param snapshot: The name or ID a backup
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the backup does not exist.
         :param bool details: When set to ``False`` no additional details will
             be returned. The default, ``True``, will cause objects with
             additional attributes to be returned.
 
         :returns: One :class:`~openstack.block_storage.v2.backup.Backup`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         :raises: :class:`~openstack.exceptions.DuplicateResource` when multiple
             resources are found.
@@ -568,7 +640,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :param backup: The value can be the ID of a backup or a
             :class:`~openstack.block_storage.v2.backup.Backup` instance
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone.
@@ -601,7 +673,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """Reset status of the backup
 
         :param backup: The value can be either the ID of a backup or a
-            :class:`~openstack.block_storage.v3.backup.Backup` instance.
+            :class:`~openstack.block_storage.v2.backup.Backup` instance.
         :param str status: New backup status
 
         :returns: None
@@ -609,6 +681,272 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         backup = self._get_resource(_backup.Backup, backup)
         backup.reset(self, status)
 
+    # ====== LIMITS ======
+    def get_limits(self, project=None):
+        """Retrieves limits
+
+        :param project: A project to get limits for. The value can be either
+            the ID of a project or an
+            :class:`~openstack.identity.v2.project.Project` instance.
+        :returns: A Limits object, including both
+            :class:`~openstack.block_storage.v2.limits.AbsoluteLimit` and
+            :class:`~openstack.block_storage.v2.limits.RateLimit`
+        :rtype: :class:`~openstack.block_storage.v2.limits.Limits`
+        """
+        params = {}
+        if project:
+            params['project_id'] = resource.Resource._get_id(project)
+        return self._get(_limits.Limits, requires_id=False, **params)
+
+    # ====== CAPABILITIES ======
+    def get_capabilities(self, host):
+        """Get a backend's capabilites
+
+        :param host: Specified backend to obtain volume stats and properties.
+
+        :returns: One :class:
+            `~openstack.block_storage.v2.capabilites.Capabilities` instance.
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            resource can be found.
+        """
+        return self._get(_capabilities.Capabilities, host)
+
+    # ====== QUOTA CLASS SETS ======
+    def get_quota_class_set(self, quota_class_set='default'):
+        """Get a single quota class set
+
+        Only one quota class is permitted, ``default``.
+
+        :param quota_class_set: The value can be the ID of a quota class set
+            (only ``default`` is supported) or a
+            :class:`~openstack.block_storage.v2.quota_class_set.QuotaClassSet`
+            instance.
+
+        :returns: One
+            :class:`~openstack.block_storage.v2.quota_class_set.QuotaClassSet`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
+            when no resource can be found.
+        """
+        return self._get(_quota_class_set.QuotaClassSet, quota_class_set)
+
+    def update_quota_class_set(self, quota_class_set, **attrs):
+        """Update a QuotaClassSet.
+
+        Only one quota class is permitted, ``default``.
+
+        :param quota_class_set: Either the ID of a quota class set (only
+            ``default`` is supported) or a
+            :class:`~openstack.block_storage.v2.quota_class_set.QuotaClassSet`
+            instance.
+        :param attrs: The attributes to update on the QuotaClassSet represented
+            by ``quota_class_set``.
+
+        :returns: The updated QuotaSet
+        :rtype: :class:`~openstack.block_storage.v2.quota_set.QuotaSet`
+        """
+        return self._update(
+            _quota_class_set.QuotaClassSet, quota_class_set, **attrs
+        )
+
+    # ====== QUOTA SETS ======
+    def get_quota_set(self, project, usage=False, **query):
+        """Show QuotaSet information for the project
+
+        :param project: ID or instance of
+            :class:`~openstack.identity.project.Project` of the project for
+            which the quota should be retrieved
+        :param bool usage: When set to ``True`` quota usage and reservations
+            would be filled.
+        :param dict query: Additional query parameters to use.
+
+        :returns: One :class:`~openstack.block_storage.v2.quota_set.QuotaSet`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
+            when no resource can be found.
+        """
+        project = self._get_resource(_project.Project, project)
+        res = self._get_resource(
+            _quota_set.QuotaSet, None, project_id=project.id
+        )
+        return res.fetch(self, usage=usage, **query)
+
+    def get_quota_set_defaults(self, project):
+        """Show QuotaSet defaults for the project
+
+        :param project: ID or instance of
+            :class:`~openstack.identity.project.Project` of the project for
+            which the quota should be retrieved
+
+        :returns: One :class:`~openstack.block_storage.v2.quota_set.QuotaSet`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
+            when no resource can be found.
+        """
+        project = self._get_resource(_project.Project, project)
+        res = self._get_resource(
+            _quota_set.QuotaSet, None, project_id=project.id
+        )
+        return res.fetch(self, base_path='/os-quota-sets/defaults')
+
+    def revert_quota_set(self, project, **query):
+        """Reset Quota for the project/user.
+
+        :param project: ID or instance of
+            :class:`~openstack.identity.project.Project` of the project for
+            which the quota should be resetted.
+        :param dict query: Additional parameters to be used.
+
+        :returns: ``None``
+        """
+        project = self._get_resource(_project.Project, project)
+        res = self._get_resource(
+            _quota_set.QuotaSet, None, project_id=project.id
+        )
+
+        if not query:
+            query = {}
+        return res.delete(self, **query)
+
+    def update_quota_set(self, project, **attrs):
+        """Update a QuotaSet.
+
+        :param project: ID or instance of
+            :class:`~openstack.identity.project.Project` of the project for
+            which the quota should be reset.
+        :param attrs: The attributes to update on the QuotaSet represented
+            by ``quota_set``.
+
+        :returns: The updated QuotaSet
+        :rtype: :class:`~openstack.block_storage.v3.quota_set.QuotaSet`
+        """
+        if 'project_id' in attrs or isinstance(project, _quota_set.QuotaSet):
+            warnings.warn(
+                "The signature of 'update_quota_set' has changed and it "
+                "now expects a Project as the first argument, in line "
+                "with the other quota set methods.",
+                os_warnings.RemovedInSDK50Warning,
+            )
+            # cinder doesn't support any query parameters so we simply pop
+            # these
+            if 'query' in attrs:
+                warnings.warn(
+                    "The query argument is no longer supported and should "
+                    "be removed.",
+                    os_warnings.RemovedInSDK50Warning,
+                )
+                attrs.pop('query')
+
+            res = self._get_resource(_quota_set.QuotaSet, project, **attrs)
+            return res.commit(self)
+        else:
+            project = self._get_resource(_project.Project, project)
+            attrs['project_id'] = project.id
+            return self._update(_quota_set.QuotaSet, None, **attrs)
+
+    # ====== VOLUME METADATA ======
+    def get_volume_metadata(self, volume):
+        """Return a dictionary of metadata for a volume
+
+        :param volume: Either the ID of a volume or a
+            :class:`~openstack.block_storage.v2.volume.Volume`.
+
+        :returns: A :class:`~openstack.block_storage.v2.volume.Volume` with the
+            volume's metadata. All keys and values are Unicode text.
+        :rtype: :class:`~openstack.block_storage.v2.volume.Volume`
+        """
+        volume = self._get_resource(_volume.Volume, volume)
+        return volume.fetch_metadata(self)
+
+    def set_volume_metadata(self, volume, **metadata):
+        """Update metadata for a volume
+
+        :param volume: Either the ID of a volume or a
+            :class:`~openstack.block_storage.v2.volume.Volume`.
+        :param kwargs metadata: Key/value pairs to be updated in the volume's
+            metadata. No other metadata is modified by this call. All keys
+            and values are stored as Unicode.
+
+        :returns: A :class:`~openstack.block_storage.v2.volume.Volume` with the
+            volume's metadata. All keys and values are Unicode text.
+        :rtype: :class:`~openstack.block_storage.v2.volume.Volume`
+        """
+        volume = self._get_resource(_volume.Volume, volume)
+        return volume.set_metadata(self, metadata=metadata)
+
+    def delete_volume_metadata(self, volume, keys=None):
+        """Delete metadata for a volume
+
+        :param volume: Either the ID of a volume or a
+            :class:`~openstack.block_storage.v2.volume.Volume`.
+        :param list keys: The keys to delete. If left empty complete
+            metadata will be removed.
+
+        :rtype: ``None``
+        """
+        volume = self._get_resource(_volume.Volume, volume)
+        if keys is not None:
+            for key in keys:
+                volume.delete_metadata_item(self, key)
+        else:
+            volume.delete_metadata(self)
+
+    # ====== SNAPSHOT METADATA ======
+    def get_snapshot_metadata(self, snapshot):
+        """Return a dictionary of metadata for a snapshot
+
+        :param snapshot: Either the ID of a snapshot or a
+            :class:`~openstack.block_storage.v2.snapshot.Snapshot`.
+
+        :returns: A
+            :class:`~openstack.block_storage.v2.snapshot.Snapshot` with the
+            snapshot's metadata. All keys and values are Unicode text.
+        :rtype: :class:`~openstack.block_storage.v2.snapshot.Snapshot`
+        """
+        snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
+        return snapshot.fetch_metadata(self)
+
+    def set_snapshot_metadata(self, snapshot, **metadata):
+        """Update metadata for a snapshot
+
+        :param snapshot: Either the ID of a snapshot or a
+            :class:`~openstack.block_storage.v2.snapshot.Snapshot`.
+        :param kwargs metadata: Key/value pairs to be updated in the snapshot's
+            metadata. No other metadata is modified by this call. All keys
+            and values are stored as Unicode.
+
+        :returns: A
+            :class:`~openstack.block_storage.v2.snapshot.Snapshot` with the
+            snapshot's metadata. All keys and values are Unicode text.
+        :rtype: :class:`~openstack.block_storage.v2.snapshot.Snapshot`
+        """
+        snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
+        return snapshot.set_metadata(self, metadata=metadata)
+
+    def delete_snapshot_metadata(self, snapshot, keys=None):
+        """Delete metadata for a snapshot
+
+        :param snapshot: Either the ID of a snapshot or a
+            :class:`~openstack.block_storage.v2.snapshot.Snapshot`.
+        :param list keys: The keys to delete. If left empty complete
+            metadata will be removed.
+
+        :rtype: ``None``
+        """
+        snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
+        if keys is not None:
+            for key in keys:
+                snapshot.delete_metadata_item(self, key)
+        else:
+            snapshot.delete_metadata(self)
+
+    # ====== EXTENSIONS ======
+    def extensions(self):
+        """Return a generator of extensions
+
+        :returns: A generator of extension
+        :rtype: :class:`~openstack.block_storage.v2.extension.Extension`
+        """
+        return self._list(_extension.Extension)
+
+    # ====== UTILS ======
     def wait_for_status(
         self,
         res,
@@ -675,170 +1013,3 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             wait,
             callback=callback,
         )
-
-    def get_quota_set(self, project, usage=False, **query):
-        """Show QuotaSet information for the project
-
-        :param project: ID or instance of
-            :class:`~openstack.identity.project.Project` of the project for
-            which the quota should be retrieved
-        :param bool usage: When set to ``True`` quota usage and reservations
-            would be filled.
-        :param dict query: Additional query parameters to use.
-
-        :returns: One :class:`~openstack.block_storage.v2.quota_set.QuotaSet`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
-            when no resource can be found.
-        """
-        project = self._get_resource(_project.Project, project)
-        res = self._get_resource(
-            _quota_set.QuotaSet, None, project_id=project.id
-        )
-        return res.fetch(self, usage=usage, **query)
-
-    def get_quota_set_defaults(self, project):
-        """Show QuotaSet defaults for the project
-
-        :param project: ID or instance of
-            :class:`~openstack.identity.project.Project` of the project for
-            which the quota should be retrieved
-
-        :returns: One :class:`~openstack.block_storage.v2.quota_set.QuotaSet`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
-            when no resource can be found.
-        """
-        project = self._get_resource(_project.Project, project)
-        res = self._get_resource(
-            _quota_set.QuotaSet, None, project_id=project.id
-        )
-        return res.fetch(self, base_path='/os-quota-sets/defaults')
-
-    def revert_quota_set(self, project, **query):
-        """Reset Quota for the project/user.
-
-        :param project: ID or instance of
-            :class:`~openstack.identity.project.Project` of the project for
-            which the quota should be resetted.
-        :param dict query: Additional parameters to be used.
-
-        :returns: ``None``
-        """
-        project = self._get_resource(_project.Project, project)
-        res = self._get_resource(
-            _quota_set.QuotaSet, None, project_id=project.id
-        )
-
-        if not query:
-            query = {}
-        return res.delete(self, **query)
-
-    def update_quota_set(self, quota_set, query=None, **attrs):
-        """Update a QuotaSet.
-
-        :param quota_set: Either the ID of a quota_set or a
-            :class:`~openstack.block_storage.v2.quota_set.QuotaSet` instance.
-        :param dict query: Optional parameters to be used with update call.
-        :param attrs: The attributes to update on the QuotaSet represented
-            by ``quota_set``.
-
-        :returns: The updated QuotaSet
-        :rtype: :class:`~openstack.block_storage.v2.quota_set.QuotaSet`
-        """
-        res = self._get_resource(_quota_set.QuotaSet, quota_set, **attrs)
-        if not query:
-            query = {}
-        return res.commit(self, **query)
-
-    def get_volume_metadata(self, volume):
-        """Return a dictionary of metadata for a volume
-
-        :param volume: Either the ID of a volume or a
-            :class:`~openstack.block_storage.v2.volume.Volume`.
-
-        :returns: A :class:`~openstack.block_storage.v2.volume.Volume` with the
-            volume's metadata. All keys and values are Unicode text.
-        :rtype: :class:`~openstack.block_storage.v2.volume.Volume`
-        """
-        volume = self._get_resource(_volume.Volume, volume)
-        return volume.fetch_metadata(self)
-
-    def set_volume_metadata(self, volume, **metadata):
-        """Update metadata for a volume
-
-        :param volume: Either the ID of a volume or a
-            :class:`~openstack.block_storage.v2.volume.Volume`.
-        :param kwargs metadata: Key/value pairs to be updated in the volume's
-            metadata. No other metadata is modified by this call. All keys
-            and values are stored as Unicode.
-
-        :returns: A :class:`~openstack.block_storage.v2.volume.Volume` with the
-            volume's metadata. All keys and values are Unicode text.
-        :rtype: :class:`~openstack.block_storage.v2.volume.Volume`
-        """
-        volume = self._get_resource(_volume.Volume, volume)
-        return volume.set_metadata(self, metadata=metadata)
-
-    def delete_volume_metadata(self, volume, keys=None):
-        """Delete metadata for a volume
-
-        :param volume: Either the ID of a volume or a
-            :class:`~openstack.block_storage.v2.volume.Volume`.
-        :param list keys: The keys to delete. If left empty complete
-            metadata will be removed.
-
-        :rtype: ``None``
-        """
-        volume = self._get_resource(_volume.Volume, volume)
-        if keys is not None:
-            for key in keys:
-                volume.delete_metadata_item(self, key)
-        else:
-            volume.delete_metadata(self)
-
-    def get_snapshot_metadata(self, snapshot):
-        """Return a dictionary of metadata for a snapshot
-
-        :param snapshot: Either the ID of a snapshot or a
-            :class:`~openstack.block_storage.v2.snapshot.Snapshot`.
-
-        :returns: A
-            :class:`~openstack.block_storage.v2.snapshot.Snapshot` with the
-            snapshot's metadata. All keys and values are Unicode text.
-        :rtype: :class:`~openstack.block_storage.v2.snapshot.Snapshot`
-        """
-        snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
-        return snapshot.fetch_metadata(self)
-
-    def set_snapshot_metadata(self, snapshot, **metadata):
-        """Update metadata for a snapshot
-
-        :param snapshot: Either the ID of a snapshot or a
-            :class:`~openstack.block_storage.v2.snapshot.Snapshot`.
-        :param kwargs metadata: Key/value pairs to be updated in the snapshot's
-            metadata. No other metadata is modified by this call. All keys
-            and values are stored as Unicode.
-
-        :returns: A
-            :class:`~openstack.block_storage.v2.snapshot.Snapshot` with the
-            snapshot's metadata. All keys and values are Unicode text.
-        :rtype: :class:`~openstack.block_storage.v2.snapshot.Snapshot`
-        """
-        snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
-        return snapshot.set_metadata(self, metadata=metadata)
-
-    def delete_snapshot_metadata(self, snapshot, keys=None):
-        """Delete metadata for a snapshot
-
-        :param snapshot: Either the ID of a snapshot or a
-            :class:`~openstack.block_storage.v2.snapshot.Snapshot`.
-        :param list keys: The keys to delete. If left empty complete
-            metadata will be removed.
-
-        :rtype: ``None``
-        """
-        snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
-        if keys is not None:
-            for key in keys:
-                snapshot.delete_metadata_item(self, key)
-        else:
-            snapshot.delete_metadata(self)
