@@ -9,6 +9,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
+import typing as ty
+
 from openstack import exceptions
 from openstack import resource
 from openstack import utils
@@ -25,15 +28,17 @@ class Backup(resource.Resource):
     # search (name~, status~, volume_id~). But this is not documented
     # officially and seem to require microversion be set
     _query_mapping = resource.QueryParameters(
-        'all_tenants',
-        'limit',
-        'marker',
-        'project_id',
-        'name',
-        'status',
-        'volume_id',
-        'sort_key',
-        'sort_dir',
+        "limit",
+        "marker",
+        "offset",
+        "project_id",
+        "name",
+        "status",
+        "volume_id",
+        "sort_key",
+        "sort_dir",
+        "sort",
+        all_projects="all_tenants",
     )
 
     # capabilities
@@ -95,6 +100,8 @@ class Backup(resource.Resource):
     user_id = resource.Body('user_id')
     #: The UUID of the volume.
     volume_id = resource.Body("volume_id")
+    #: The name of the volume.
+    volume_name = resource.Body("volume_name")
 
     _max_microversion = "3.64"
 
@@ -153,7 +160,7 @@ class Backup(resource.Resource):
         else:
             # Just for safety of the implementation (since PUT removed)
             raise exceptions.ResourceFailure(
-                "Invalid create method: %s" % self.create_method
+                f"Invalid create method: {self.create_method}"
             )
 
         has_body = (
@@ -187,22 +194,22 @@ class Backup(resource.Resource):
         :return: Updated backup instance
         """
         url = utils.urljoin(self.base_path, self.id, "restore")
-        body = {'restore': {}}
+        body: ty.Dict[str, ty.Dict] = {'restore': {}}
         if volume_id:
             body['restore']['volume_id'] = volume_id
         if name:
             body['restore']['name'] = name
         if not (volume_id or name):
             raise exceptions.SDKException(
-                'Either of `name` or `volume_id`' ' must be specified.'
+                'Either of `name` or `volume_id` must be specified.'
             )
         response = session.post(url, json=body)
-        self._translate_response(response, has_body=False)
+        self._translate_response(response, resource_response_key='restore')
         return self
 
     def force_delete(self, session):
         """Force backup deletion"""
-        body = {'os-force_delete': {}}
+        body = {'os-force_delete': None}
         self._action(session, body)
 
     def reset(self, session, status):
