@@ -10,40 +10,34 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import atexit
+import concurrent.futures
 import copy
 import functools
 import queue
+import typing as ty
 import warnings
+import weakref
 
 import dogpile.cache
 import keystoneauth1.exceptions
-import keystoneauth1.session
 import requests.models
 import requestsexceptions
 
 from openstack import _log
-from openstack.cloud import _floating_ip
-from openstack.cloud import _object_store
+from openstack import _services_mixin
 from openstack.cloud import _utils
-from openstack.cloud import exc
 from openstack.cloud import meta
 import openstack.config
-from openstack.config import cloud_region as cloud_region_mod
+import openstack.config.cloud_region
 from openstack import exceptions
 from openstack import proxy
+from openstack import resource
 from openstack import utils
-
-DEFAULT_SERVER_AGE = 5
-DEFAULT_PORT_AGE = 5
-DEFAULT_FLOAT_AGE = 5
-_CONFIG_DOC_URL = _floating_ip._CONFIG_DOC_URL
-DEFAULT_OBJECT_SEGMENT_SIZE = _object_store.DEFAULT_OBJECT_SEGMENT_SIZE
-# This halves the current default for Swift
-DEFAULT_MAX_FILE_SIZE = _object_store.DEFAULT_MAX_FILE_SIZE
-OBJECT_CONTAINER_ACLS = _object_store.OBJECT_CONTAINER_ACLS
+from openstack import warnings as os_warnings
 
 
-class _OpenStackCloudMixin:
+class _OpenStackCloudMixin(_services_mixin.ServicesMixin):
     """Represent a connection to an OpenStack Cloud.
 
     OpenStackCloud is the entry point for all cloud operations, regardless
@@ -52,9 +46,6 @@ class _OpenStackCloudMixin:
     REST API operation oriented. For instance, one will request a Floating IP
     and that Floating IP will be actualized either via neutron or via nova
     depending on how this particular cloud has decided to arrange itself.
-
-    :param bool strict: Only return documented attributes for each resource
-                        as per the Data Model contract. (Default False)
     """
 
     _OBJECT_MD5_KEY = 'x-sdk-md5'
@@ -69,8 +60,151 @@ class _OpenStackCloudMixin:
     _SHADE_OBJECT_SHA256_KEY = 'x-object-meta-x-shade-sha256'
     _SHADE_OBJECT_AUTOCREATE_KEY = 'x-object-meta-x-shade-autocreated'
 
-    def __init__(self):
-        super(_OpenStackCloudMixin, self).__init__()
+    def __init__(
+        self,
+        cloud=None,
+        config=None,
+        session=None,
+        app_name=None,
+        app_version=None,
+        extra_services=None,
+        strict=False,
+        use_direct_get=None,
+        task_manager=None,
+        rate_limit=None,
+        oslo_conf=None,
+        service_types=None,
+        global_request_id=None,
+        strict_proxies=False,
+        pool_executor=None,
+        **kwargs,
+    ):
+        """Create a connection to a cloud.
+
+        A connection needs information about how to connect, how to
+        authenticate and how to select the appropriate services to use.
+
+        The recommended way to provide this information is by referencing
+        a named cloud config from an existing `clouds.yaml` file. The cloud
+        name ``envvars`` may be used to consume a cloud configured via ``OS_``
+        environment variables.
+
+        A pre-existing :class:`~openstack.config.cloud_region.CloudRegion`
+        object can be passed in lieu of a cloud name, for cases where the user
+        already has a fully formed CloudRegion and just wants to use it.
+
+        Similarly, if for some reason the user already has a
+        :class:`~keystoneauth1.session.Session` and wants to use it, it may be
+        passed in.
+
+        :param str cloud: Name of the cloud from config to use.
+        :param config: CloudRegion object representing the config for the
+            region of the cloud in question.
+        :type config: :class:`~openstack.config.cloud_region.CloudRegion`
+        :param session: A session object compatible with
+            :class:`~keystoneauth1.session.Session`.
+        :type session: :class:`~keystoneauth1.session.Session`
+        :param str app_name: Name of the application to be added to User Agent.
+        :param str app_version: Version of the application to be added to
+            User Agent.
+        :param extra_services: List of
+            :class:`~openstack.service_description.ServiceDescription`
+            objects describing services that openstacksdk otherwise does not
+            know about.
+        :param bool use_direct_get:
+            For get methods, make specific REST calls for server-side
+            filtering instead of making list calls and filtering client-side.
+            Default false.
+        :param task_manager:
+            Ignored. Exists for backwards compat during transition. Rate limit
+            parameters should be passed directly to the `rate_limit` parameter.
+        :param rate_limit:
+            Client-side rate limit, expressed in calls per second. The
+            parameter can either be a single float, or it can be a dict with
+            keys as service-type and values as floats expressing the calls
+            per second for that service. Defaults to None, which means no
+            rate-limiting is performed.
+        :param oslo_conf: An oslo.config CONF object.
+        :type oslo_conf: :class:`~oslo_config.cfg.ConfigOpts`
+            An oslo.config ``CONF`` object that has been populated with
+            ``keystoneauth1.loading.register_adapter_conf_options`` in
+            groups named by the OpenStack service's project name.
+        :param service_types:
+            A list/set of service types this Connection should support. All
+            other service types will be disabled (will error if used).
+            **Currently only supported in conjunction with the ``oslo_conf``
+            kwarg.**
+        :param global_request_id: A Request-id to send with all interactions.
+        :param strict_proxies:
+            If True, check proxies on creation and raise
+            ServiceDiscoveryException if the service is unavailable.
+        :type strict_proxies: bool
+            Throw an ``openstack.exceptions.ServiceDiscoveryException`` if the
+            endpoint for a given service doesn't work. This is useful for
+            OpenStack services using sdk to talk to other OpenStack services
+            where it can be expected that the deployer config is correct and
+            errors should be reported immediately.
+            Default false.
+        :param pool_executor:
+        :type pool_executor: :class:`~futurist.Executor`
+            A futurist ``Executor`` object to be used for concurrent background
+            activities. Defaults to None in which case a ThreadPoolExecutor
+            will be created if needed.
+        :param kwargs: If a config is not provided, the rest of the parameters
+            provided are assumed to be arguments to be passed to the
+            CloudRegion constructor.
+        """
+        super().__init__()
+
+        if use_direct_get is not None:
+            warnings.warn(
+                "The 'use_direct_get' argument is deprecated for removal",
+                os_warnings.RemovedInSDK50Warning,
+            )
+
+        self.config = config
+        self._extra_services = {}
+        self._strict_proxies = strict_proxies
+        if extra_services:
+            for service in extra_services:
+                self._extra_services[service.service_type] = service
+
+        if not self.config:
+            if oslo_conf:
+                self.config = openstack.config.cloud_region.from_conf(
+                    oslo_conf,
+                    session=session,
+                    app_name=app_name,
+                    app_version=app_version,
+                    service_types=service_types,
+                )
+            elif session:
+                self.config = openstack.config.cloud_region.from_session(
+                    session=session,
+                    app_name=app_name,
+                    app_version=app_version,
+                    load_yaml_config=False,
+                    load_envvars=False,
+                    rate_limit=rate_limit,
+                    **kwargs,
+                )
+            else:
+                self.config = openstack.config.get_cloud_region(
+                    cloud=cloud,
+                    app_name=app_name,
+                    app_version=app_version,
+                    load_yaml_config=cloud is not None,
+                    load_envvars=cloud is not None,
+                    rate_limit=rate_limit,
+                    **kwargs,
+                )
+
+        self._session = None
+        self._proxies = {}
+        self.__pool_executor = pool_executor
+        self._global_request_id = global_request_id
+        self.use_direct_get = use_direct_get or False
+        self.strict_mode = strict
 
         self.log = _log.setup_logging('openstack')
 
@@ -106,35 +240,6 @@ class _OpenStackCloudMixin:
         else:
             self.cache_enabled = False
 
-            # TODO(gtema): delete it with the standalone cloud layer caching
-
-            def _fake_invalidate(unused):
-                pass
-
-            class _FakeCache:
-                def invalidate(self):
-                    pass
-
-            # Don't cache list_servers if we're not caching things.
-            # Replace this with a more specific cache configuration
-            # soon.
-            self._SERVER_AGE = 0
-            self._PORT_AGE = 0
-            self._FLOAT_AGE = 0
-            self._cache = _FakeCache()
-            # Undecorate cache decorated methods. Otherwise the call stacks
-            # wind up being stupidly long and hard to debug
-            for method in _utils._decorated_methods:
-                meth_obj = getattr(self, method, None)
-                if not meth_obj:
-                    continue
-                if hasattr(meth_obj, 'invalidate') and hasattr(
-                    meth_obj, 'func'
-                ):
-                    new_func = functools.partial(meth_obj.func, self)
-                    new_func.invalidate = _fake_invalidate
-                    setattr(self, method, new_func)
-
         # Uncoditionally create cache even with a "null" backend
         self._cache = self._make_cache(
             cache_class, cache_expiration_time, cache_arguments
@@ -143,124 +248,50 @@ class _OpenStackCloudMixin:
         for expire_key in expirations.keys():
             self._cache_expirations[expire_key] = expirations[expire_key]
 
-        # TODO(gtema): delete in next change
-        self._SERVER_AGE = 0
-        self._PORT_AGE = 0
-        self._FLOAT_AGE = 0
-
         self._api_cache_keys = set()
         self._container_cache = dict()
         self._file_hash_cache = dict()
 
-        # self.__pool_executor = None
-
-        self._raw_clients = {}
-
         self._local_ipv6 = (
             _utils.localhost_supports_ipv6() if not self.force_ipv4 else False
         )
 
-    def connect_as(self, **kwargs):
-        """Make a new OpenStackCloud object with new auth context.
+        # Register cleanup steps
+        atexit.register(self.close)
 
-        Take the existing settings from the current cloud and construct a new
-        OpenStackCloud object with some of the auth settings overridden. This
-        is useful for getting an object to perform tasks with as another user,
-        or in the context of a different project.
+    @property
+    def session(self):
+        if not self._session:
+            self._session = self.config.get_session()
+            # Hide a reference to the connection on the session to help with
+            # backwards compatibility for folks trying to just pass
+            # conn.session to a Resource method's session argument.
+            self.session._sdk_connection = weakref.proxy(self)
+        return self._session
 
-        .. code-block:: python
-
-          conn = openstack.connect(cloud='example')
-          # Work normally
-          servers = conn.list_servers()
-          conn2 = conn.connect_as(username='different-user', password='')
-          # Work as different-user
-          servers = conn2.list_servers()
-
-        :param kwargs: keyword arguments can contain anything that would
-                       normally go in an auth dict. They will override the same
-                       settings from the parent cloud as appropriate. Entries
-                       that do not want to be overridden can be ommitted.
-        """
-
-        if self.config._openstack_config:
-            config = self.config._openstack_config
-        else:
-            # TODO(mordred) Replace this with from_session
-            config = openstack.config.OpenStackConfig(
-                app_name=self.config._app_name,
-                app_version=self.config._app_version,
-                load_yaml_config=False,
+    @property
+    def _pool_executor(self):
+        if not self.__pool_executor:
+            self.__pool_executor = concurrent.futures.ThreadPoolExecutor(
+                max_workers=5
             )
-        params = copy.deepcopy(self.config.config)
-        # Remove profile from current cloud so that overridding works
-        params.pop('profile', None)
+        return self.__pool_executor
 
-        # Utility function to help with the stripping below.
-        def pop_keys(params, auth, name_key, id_key):
-            if name_key in auth or id_key in auth:
-                params['auth'].pop(name_key, None)
-                params['auth'].pop(id_key, None)
+    def close(self):
+        """Release any resources held open."""
+        self.config.set_auth_cache()
+        if self.__pool_executor:
+            self.__pool_executor.shutdown()
+        atexit.unregister(self.close)
 
-        # If there are user, project or domain settings in the incoming auth
-        # dict, strip out both id and name so that a user can say:
-        #     cloud.connect_as(project_name='foo')
-        # and have that work with clouds that have a project_id set in their
-        # config.
-        for prefix in ('user', 'project'):
-            if prefix == 'user':
-                name_key = 'username'
-            else:
-                name_key = 'project_name'
-            id_key = '{prefix}_id'.format(prefix=prefix)
-            pop_keys(params, kwargs, name_key, id_key)
-            id_key = '{prefix}_domain_id'.format(prefix=prefix)
-            name_key = '{prefix}_domain_name'.format(prefix=prefix)
-            pop_keys(params, kwargs, name_key, id_key)
+    def __enter__(self):
+        return self
 
-        for key, value in kwargs.items():
-            params['auth'][key] = value
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
 
-        cloud_region = config.get_one(**params)
-        # Attach the discovery cache from the old session so we won't
-        # double discover.
-        cloud_region._discovery_cache = self.session._discovery_cache
-        # Override the cloud name so that logging/location work right
-        cloud_region._name = self.name
-        cloud_region.config['profile'] = self.name
-        # Use self.__class__ so that we return whatever this if, like if it's
-        # a subclass in the case of shade wrapping sdk.
-        return self.__class__(config=cloud_region)
-
-    def connect_as_project(self, project):
-        """Make a new OpenStackCloud object with a new project.
-
-        Take the existing settings from the current cloud and construct a new
-        OpenStackCloud object with the project settings overridden. This
-        is useful for getting an object to perform tasks with as another user,
-        or in the context of a different project.
-
-        .. code-block:: python
-
-          cloud = openstack.connect(cloud='example')
-          # Work normally
-          servers = cloud.list_servers()
-          cloud2 = cloud.connect_as_project('different-project')
-          # Work in different-project
-          servers = cloud2.list_servers()
-
-        :param project: Either a project name or a project dict as returned by
-                        `list_projects`.
-        """
-        auth = {}
-        if isinstance(project, dict):
-            auth['project_id'] = project.get('id')
-            auth['project_name'] = project.get('name')
-            if project.get('domain_id'):
-                auth['project_domain_id'] = project['domain_id']
-        else:
-            auth['project_name'] = project
-        return self.connect_as(**auth)
+    def set_global_request_id(self, global_request_id):
+        self._global_request_id = global_request_id
 
     def global_request(self, global_request_id):
         """Make a new Connection object with a global request id set.
@@ -270,30 +301,32 @@ class _OpenStackCloudMixin:
 
         .. code-block:: python
 
-          from oslo_context import context
-          cloud = openstack.connect(cloud='example')
-          # Work normally
-          servers = cloud.list_servers()
-          cloud2 = cloud.global_request(context.generate_request_id())
-          # cloud2 sends all requests with global_request_id set
-          servers = cloud2.list_servers()
+            from oslo_context import context
+
+            cloud = openstack.connect(cloud='example')
+            # Work normally
+            servers = cloud.list_servers()
+            cloud2 = cloud.global_request(context.generate_request_id())
+            # cloud2 sends all requests with global_request_id set
+            servers = cloud2.list_servers()
 
         Additionally, this can be used as a context manager:
 
         .. code-block:: python
 
-          from oslo_context import context
-          c = openstack.connect(cloud='example')
-          # Work normally
-          servers = c.list_servers()
-          with c.global_request(context.generate_request_id()) as c2:
-              # c2 sends all requests with global_request_id set
-              servers = c2.list_servers()
+            from oslo_context import context
+
+            c = openstack.connect(cloud='example')
+            # Work normally
+            servers = c.list_servers()
+            with c.global_request(context.generate_request_id()) as c2:
+                # c2 sends all requests with global_request_id set
+                servers = c2.list_servers()
 
         :param global_request_id: The `global_request_id` to send.
         """
         params = copy.deepcopy(self.config.config)
-        cloud_region = cloud_region_mod.from_session(
+        cloud_region = openstack.config.cloud_region.from_session(
             session=self.session,
             app_name=self.config._app_name,
             app_version=self.config._app_version,
@@ -322,41 +355,20 @@ class _OpenStackCloudMixin:
         if namespace is None:
             name_key = self.name
         else:
-            name_key = '%s:%s' % (self.name, namespace)
+            name_key = f'{self.name}:{namespace}'
 
         def generate_key(*args, **kwargs):
             # TODO(frickler): make handling arg keys actually work
             arg_key = ''
             kw_keys = sorted(kwargs.keys())
             kwargs_key = ','.join(
-                ['%s:%s' % (k, kwargs[k]) for k in kw_keys if k != 'cache']
+                [f'{k}:{kwargs[k]}' for k in kw_keys if k != 'cache']
             )
             ans = "_".join([str(name_key), fname, arg_key, kwargs_key])
             return ans
 
         return generate_key
 
-    def _get_cache(self, resource_name):
-        if resource_name and resource_name in self._resource_caches:
-            return self._resource_caches[resource_name]
-        else:
-            return self._cache
-
-    # TODO(shade) This should be replaced with using openstack Connection
-    #             object.
-    def _get_raw_client(
-        self, service_type, api_version=None, endpoint_override=None
-    ):
-        return proxy.Proxy(
-            session=self.session,
-            service_type=self.config.get_service_type(service_type),
-            service_name=self.config.get_service_name(service_type),
-            interface=self.config.get_interface(service_type),
-            endpoint_override=self.config.get_endpoint(service_type)
-            or endpoint_override,
-            region_name=self.config.get_region_name(service_type),
-        )
-
     def pprint(self, resource):
         """Wrapper around pprint that groks munch objects"""
         # import late since this is a utility function
@@ -381,34 +393,6 @@ class _OpenStackCloudMixin:
     def service_catalog(self):
         return self._keystone_catalog.catalog
 
-    def endpoint_for(self, service_type, interface=None, region_name=None):
-        """Return the endpoint for a given service.
-
-        Respects config values for Connection, including
-        ``*_endpoint_override``. For direct values from the catalog
-        regardless of overrides, see
-        :meth:`~openstack.config.cloud_region.CloudRegion.get_endpoint_from_catalog`
-
-        :param service_type: Service Type of the endpoint to search for.
-        :param interface:
-            Interface of the endpoint to search for. Optional, defaults to
-            the configured value for interface for this Connection.
-        :param region_name:
-            Region Name of the endpoint to search for. Optional, defaults to
-            the configured value for region_name for this Connection.
-
-        :returns: The endpoint of the service, or None if not found.
-        """
-
-        endpoint_override = self.config.get_endpoint(service_type)
-        if endpoint_override:
-            return endpoint_override
-        return self.config.get_endpoint_from_catalog(
-            service_type=service_type,
-            interface=interface,
-            region_name=region_name,
-        )
-
     @property
     def auth_token(self):
         # Keystone's session will reuse a token if it is still valid.
@@ -479,18 +463,6 @@ class _OpenStackCloudMixin:
             project=self._get_project_info(project_id),
         )
 
-    # TODO(stephenfin): This looks unused? Can we delete it?
-    def _get_identity_location(self):
-        '''Identity resources do not exist inside of projects.'''
-        return utils.Munch(
-            cloud=self.name,
-            region_name=None,
-            zone=None,
-            project=utils.Munch(
-                id=None, name=None, domain_id=None, domain_name=None
-            ),
-        )
-
     def range_search(self, data, filters):
         """Perform integer range searches across a list of dictionaries.
 
@@ -517,9 +489,10 @@ class _OpenStackCloudMixin:
                 {"vcpus": "<=5", "ram": "<=2048", "disk": "1"}
 
         :returns: A list subset of the original data set.
-        :raises: OpenStackCloudException on invalid range expressions.
+        :raises: :class:`~openstack.exceptions.SDKException` on invalid range
+            expressions.
         """
-        filtered = []
+        filtered: ty.List[object] = []
 
         for key, range_value in filters.items():
             # We always want to operate on the full data set so that
@@ -563,17 +536,12 @@ class _OpenStackCloudMixin:
                 "Endpoint not found in %s cloud: %s", self.name, str(e)
             )
             endpoint = None
-        except exc.OpenStackCloudException:
+        except exceptions.SDKException:
             raise
         except Exception as e:
-            raise exc.OpenStackCloudException(
-                "Error getting {service} endpoint on {cloud}:{region}:"
-                " {error}".format(
-                    service=service_key,
-                    cloud=self.name,
-                    region=self.config.get_region_name(service_key),
-                    error=str(e),
-                )
+            raise exceptions.SDKException(
+                f"Error getting {service_key} endpoint on {self.name}:{self.config.get_region_name(service_key)}: "
+                f"{str(e)}"
             )
         return endpoint
 
@@ -597,7 +565,7 @@ class _OpenStackCloudMixin:
                 kwargs['min_version'] = version
                 kwargs['max_version'] = version
             endpoint = self.get_session_endpoint(service_key, **kwargs)
-        except exc.OpenStackCloudException:
+        except exceptions.SDKException:
             return False
         if endpoint:
             return True
@@ -638,15 +606,14 @@ class _OpenStackCloudMixin:
         (service_name, resource_name) = resource_type.split('.')
         if not hasattr(self, service_name):
             raise exceptions.SDKException(
-                "service %s is not existing/enabled" % service_name
+                f"service {service_name} is not existing/enabled"
             )
         service_proxy = getattr(self, service_name)
         try:
             resource_type = service_proxy._resource_registry[resource_name]
         except KeyError:
             raise exceptions.SDKException(
-                "Resource %s is not known in service %s"
-                % (resource_name, service_name)
+                f"Resource {resource_name} is not known in service {service_name}"
             )
 
         if name_or_id:
@@ -656,7 +623,7 @@ class _OpenStackCloudMixin:
                     resource_type, name_or_id, *get_args, **get_kwargs
                 )
                 return [resource_by_id]
-            except exceptions.ResourceNotFound:
+            except exceptions.NotFoundException:
                 pass
 
         if not filters:
@@ -730,7 +697,7 @@ class _OpenStackCloudMixin:
             for dep in v.get('after', []):
                 dep_graph.add_edge(dep, k)
 
-        cleanup_resources = dict()
+        cleanup_resources: ty.Dict[str, resource.Resource] = {}
 
         for service in dep_graph.walk(timeout=wait_timeout):
             fn = None
@@ -772,6 +739,6 @@ def cleanup_task(graph, service, fn):
         fn()
     except Exception:
         log = _log.setup_logging('openstack.project_cleanup')
-        log.exception('Error in the %s cleanup function' % service)
+        log.exception(f'Error in the {service} cleanup function')
     finally:
         graph.node_done(service)
