@@ -11,15 +11,31 @@
 # under the License.
 
 from openstack.dns.v2 import floating_ip as _fip
+from openstack.dns.v2 import limit as _limit
 from openstack.dns.v2 import recordset as _rs
+from openstack.dns.v2 import service_status as _svc_status
+from openstack.dns.v2 import tsigkey as _tsigkey
 from openstack.dns.v2 import zone as _zone
 from openstack.dns.v2 import zone_export as _zone_export
 from openstack.dns.v2 import zone_import as _zone_import
+from openstack.dns.v2 import zone_share as _zone_share
 from openstack.dns.v2 import zone_transfer as _zone_transfer
 from openstack import proxy
 
 
 class Proxy(proxy.Proxy):
+    _resource_registry = {
+        "floating_ip": _fip.FloatingIP,
+        "limits": _limit.Limit,
+        "recordset": _rs.Recordset,
+        "service_status": _svc_status.ServiceStatus,
+        "zone": _zone.Zone,
+        "tsigkey": _tsigkey.TSIGKey,
+        "zone_export": _zone_export.ZoneExport,
+        "zone_import": _zone_import.ZoneImport,
+        "zone_share": _zone_share.ZoneShare,
+        "zone_transfer_request": _zone_transfer.ZoneTransferRequest,
+    }
 
     # ======== Zones ========
     def zones(self, **query):
@@ -61,21 +77,28 @@ class Proxy(proxy.Proxy):
         """
         return self._get(_zone.Zone, zone)
 
-    def delete_zone(self, zone, ignore_missing=True):
+    def delete_zone(self, zone, ignore_missing=True, delete_shares=False):
         """Delete a zone
 
         :param zone: The value can be the ID of a zone
             or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone.
+        :param bool delete_shares: When True, delete the zone shares along with
+                                   the zone.
 
         :returns: Zone been deleted
         :rtype: :class:`~openstack.dns.v2.zone.Zone`
         """
-        return self._delete(_zone.Zone, zone, ignore_missing=ignore_missing)
+        return self._delete(
+            _zone.Zone,
+            zone,
+            ignore_missing=ignore_missing,
+            delete_shares=delete_shares,
+        )
 
     def update_zone(self, zone, **attrs):
         """Update zone attributes
@@ -89,20 +112,21 @@ class Proxy(proxy.Proxy):
         """
         return self._update(_zone.Zone, zone, **attrs)
 
-    def find_zone(self, name_or_id, ignore_missing=True, **attrs):
+    def find_zone(self, name_or_id, ignore_missing=True):
         """Find a single zone
 
         :param name_or_id: The name or ID of a zone
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the zone does not exist.
             When set to ``True``, no exception will be set when attempting
             to delete a nonexistent zone.
 
         :returns: :class:`~openstack.dns.v2.zone.Zone`
         """
-        return self._find(_zone.Zone, name_or_id,
-                          ignore_missing=ignore_missing)
+        return self._find(
+            _zone.Zone, name_or_id, ignore_missing=ignore_missing
+        )
 
     def abandon_zone(self, zone, **attrs):
         """Abandon Zone
@@ -203,7 +227,7 @@ class Proxy(proxy.Proxy):
         :param zone: The value can be the ID of a zone
             or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist. When set to ``True``, no exception will
             be set when attempting to delete a nonexistent zone.
 
@@ -212,19 +236,19 @@ class Proxy(proxy.Proxy):
         """
         if zone:
             zone = self._get_resource(_zone.Zone, zone)
-            recordset = self._get(
-                _rs.Recordset, recordset, zone_id=zone.id)
-        return self._delete(_rs.Recordset, recordset,
-                            ignore_missing=ignore_missing)
+            recordset = self._get(_rs.Recordset, recordset, zone_id=zone.id)
+        return self._delete(
+            _rs.Recordset, recordset, ignore_missing=ignore_missing
+        )
 
-    def find_recordset(self, zone, name_or_id, ignore_missing=True, **attrs):
+    def find_recordset(self, zone, name_or_id, ignore_missing=True, **query):
         """Find a single recordset
 
         :param zone: The value can be the ID of a zone
             or a :class:`~openstack.dns.v2.zone.Zone` instance.
         :param name_or_id: The name or ID of a zone
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            :class:`~openstack.exceptions.NotFoundException` will be raised
             when the zone does not exist.
             When set to ``True``, no exception will be set when attempting
             to delete a nonexistent zone.
@@ -232,9 +256,13 @@ class Proxy(proxy.Proxy):
         :returns: :class:`~openstack.dns.v2.recordset.Recordset`
         """
         zone = self._get_resource(_zone.Zone, zone)
-        return self._find(_rs.Recordset, name_or_id,
-                          ignore_missing=ignore_missing, zone_id=zone.id,
-                          **attrs)
+        return self._find(
+            _rs.Recordset,
+            name_or_id,
+            ignore_missing=ignore_missing,
+            zone_id=zone.id,
+            **query,
+        )
 
     # ======== Zone Imports ========
     def zone_imports(self, **query):
@@ -261,8 +289,9 @@ class Proxy(proxy.Proxy):
         :returns: The results of zone creation.
         :rtype: :class:`~openstack.dns.v2.zone_import.ZoneImport`
         """
-        return self._create(_zone_import.ZoneImport, prepend_key=False,
-                            **attrs)
+        return self._create(
+            _zone_import.ZoneImport, prepend_key=False, **attrs
+        )
 
     def get_zone_import(self, zone_import):
         """Get a zone import record
@@ -280,15 +309,16 @@ class Proxy(proxy.Proxy):
         :param zone_import: The value can be the ID of a zone import
             or a :class:`~openstack.dns.v2.zone_import.ZoneImport` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone.
 
         :returns: None
         """
-        return self._delete(_zone_import.ZoneImport, zone_import,
-                            ignore_missing=ignore_missing)
+        return self._delete(
+            _zone_import.ZoneImport, zone_import, ignore_missing=ignore_missing
+        )
 
     # ======== Zone Exports ========
     def zone_exports(self, **query):
@@ -318,11 +348,13 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.dns.v2.zone_export.ZoneExport`
         """
         zone = self._get_resource(_zone.Zone, zone)
-        return self._create(_zone_export.ZoneExport,
-                            base_path='/zones/%(zone_id)s/tasks/export',
-                            prepend_key=False,
-                            zone_id=zone.id,
-                            **attrs)
+        return self._create(
+            _zone_export.ZoneExport,
+            base_path='/zones/%(zone_id)s/tasks/export',
+            prepend_key=False,
+            zone_id=zone.id,
+            **attrs,
+        )
 
     def get_zone_export(self, zone_export):
         """Get a zone export record
@@ -342,8 +374,11 @@ class Proxy(proxy.Proxy):
         :returns: ZoneExport instance.
         :rtype: :class:`~openstack.dns.v2.zone_export.ZoneExport`
         """
-        return self._get(_zone_export.ZoneExport, zone_export,
-                         base_path='/zones/tasks/export/%(id)s/export')
+        return self._get(
+            _zone_export.ZoneExport,
+            zone_export,
+            base_path='/zones/tasks/export/%(id)s/export',
+        )
 
     def delete_zone_export(self, zone_export, ignore_missing=True):
         """Delete a zone export
@@ -351,15 +386,16 @@ class Proxy(proxy.Proxy):
         :param zone_export: The value can be the ID of a zone import
             or a :class:`~openstack.dns.v2.zone_export.ZoneExport` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone.
 
         :returns: None
         """
-        return self._delete(_zone_export.ZoneExport, zone_export,
-                            ignore_missing=ignore_missing)
+        return self._delete(
+            _zone_export.ZoneExport, zone_export, ignore_missing=ignore_missing
+        )
 
     # ======== FloatingIPs ========
     def floating_ips(self, **query):
@@ -456,7 +492,8 @@ class Proxy(proxy.Proxy):
             base_path='/zones/%(zone_id)s/tasks/transfer_requests',
             prepend_key=False,
             zone_id=zone.id,
-            **attrs)
+            **attrs,
+        )
 
     def update_zone_transfer_request(self, request, **attrs):
         """Update ZoneTransfer Request attributes
@@ -468,8 +505,9 @@ class Proxy(proxy.Proxy):
 
         :rtype: :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
         """
-        return self._update(_zone_transfer.ZoneTransferRequest,
-                            request, **attrs)
+        return self._update(
+            _zone_transfer.ZoneTransferRequest, request, **attrs
+        )
 
     def delete_zone_transfer_request(self, request, ignore_missing=True):
         """Delete a ZoneTransfer Request
@@ -478,15 +516,18 @@ class Proxy(proxy.Proxy):
             or a :class:`~openstack.dns.v2.zone_transfer.ZoneTransferRequest`
             instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the zone does not exist.
             When set to ``True``, no exception will be set when attempting to
             delete a nonexistent zone.
 
         :returns: None
         """
-        return self._delete(_zone_transfer.ZoneTransferRequest, request,
-                            ignore_missing=ignore_missing)
+        return self._delete(
+            _zone_transfer.ZoneTransferRequest,
+            request,
+            ignore_missing=ignore_missing,
+        )
 
     def zone_transfer_accepts(self, **query):
         """Retrieve a generator of zone transfer accepts
@@ -524,35 +565,243 @@ class Proxy(proxy.Proxy):
         """
         return self._create(_zone_transfer.ZoneTransferAccept, **attrs)
 
+    # ======== Zone Shares ========
+    def zone_shares(self, zone, **query):
+        """Retrieve a generator of zone shares
+
+        :param zone: The zone ID or a
+            :class:`~openstack.dns.v2.zone.Zone` instance
+        :param dict query: Optional query parameters to be sent to limit the
+                           resources being returned.
+
+                           * `target_project_id`: The target project ID field.
+
+        :returns: A generator of zone shares
+            :class:`~openstack.dns.v2.zone_share.ZoneShare` instances.
+        """
+        zone_obj = self._get_resource(_zone.Zone, zone)
+        return self._list(_zone_share.ZoneShare, zone_id=zone_obj.id, **query)
+
+    def get_zone_share(self, zone, zone_share):
+        """Get a zone share
+
+        :param zone: The value can be the ID of a zone
+            or a :class:`~openstack.dns.v2.zone.Zone` instance.
+        :param zone_share: The zone_share can be either the ID of the zone
+            share or a :class:`~openstack.dns.v2.zone_share.ZoneShare` instance
+            that the zone share belongs to.
+
+        :returns: ZoneShare instance.
+        :rtype: :class:`~openstack.dns.v2.zone_share.ZoneShare`
+        """
+        zone_obj = self._get_resource(_zone.Zone, zone)
+        return self._get(
+            _zone_share.ZoneShare, zone_share, zone_id=zone_obj.id
+        )
+
+    def find_zone_share(self, zone, zone_share_id, ignore_missing=True):
+        """Find a single zone share
+
+        :param zone: The value can be the ID of a zone
+            or a :class:`~openstack.dns.v2.zone.Zone` instance.
+        :param zone_share_id: The zone share ID
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the zone share does not exist.
+            When set to ``True``,  None will be returned when attempting to
+            find a nonexistent zone share.
+
+        :returns: :class:`~openstack.dns.v2.zone_share.ZoneShare`
+        """
+        zone_obj = self._get_resource(_zone.Zone, zone)
+        return self._find(
+            _zone_share.ZoneShare,
+            zone_share_id,
+            ignore_missing=ignore_missing,
+            zone_id=zone_obj.id,
+        )
+
+    def create_zone_share(self, zone, **attrs):
+        """Create a new zone share from attributes
+
+        :param zone: The zone ID or a
+            :class:`~openstack.dns.v2.zone.Zone` instance
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.dns.v2.zone_share.ZoneShare`,
+            comprised of the properties on the ZoneShare class.
+
+        :returns: The results of zone share creation
+        :rtype: :class:`~openstack.dns.v2.zone_share.ZoneShare`
+        """
+        zone_obj = self._get_resource(_zone.Zone, zone)
+        return self._create(
+            _zone_share.ZoneShare, zone_id=zone_obj.id, **attrs
+        )
+
+    def delete_zone_share(self, zone, zone_share, ignore_missing=True):
+        """Delete a zone share
+
+        :param zone: The zone ID or a
+            :class:`~openstack.dns.v2.zone.Zone` instance
+        :param zone_share: The zone_share can be either the ID of the zone
+            share or a :class:`~openstack.dns.v2.zone_share.ZoneShare` instance
+            that the zone share belongs to.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
+            the zone share does not exist.
+            When set to ``True``, no exception will be set when attempting to
+            delete a nonexistent zone share.
+
+        :returns: ``None``
+        """
+        zone_obj = self._get_resource(_zone.Zone, zone)
+        self._delete(
+            _zone_share.ZoneShare,
+            zone_share,
+            ignore_missing=ignore_missing,
+            zone_id=zone_obj.id,
+        )
+
+    # ======== Limits ========
+    def limits(self, **query):
+        """Retrieve a generator of limits
+
+        :returns: A generator of limits
+            (:class:`~openstack.dns.v2.limit.Limit`) instances
+        """
+        return self._list(_limit.Limit, **query)
+
+    # ======== Service Statuses ========
+    def service_statuses(self):
+        """Retrieve a generator of service statuses
+
+        :returns: A generator of service statuses
+            :class:`~openstack.dns.v2.service_status.ServiceStatus` instances.
+        """
+        return self._list(_svc_status.ServiceStatus)
+
+    def get_service_status(self, service):
+        """Get a status of a service in the Designate system
+
+        :param service: The value can be the ID of a service
+            or a :class:`~openstack.dns.v2.service_status.ServiceStatus` instance.
+
+        :returns: ServiceStatus instance.
+        :rtype: :class:`~openstack.dns.v2.service_status.ServiceStatus`
+        """
+        return self._get(_svc_status.ServiceStatus, service)
+
     def _get_cleanup_dependencies(self):
         # DNS may depend on floating ip
-        return {
-            'dns': {
-                'before': ['network']
-            }
-        }
+        return {'dns': {'before': ['network']}}
 
-    def _service_cleanup(self, dry_run=True, client_status_queue=False,
-                         identified_resources=None,
-                         filters=None, resource_evaluation_fn=None):
-        # Delete all zones
-        for obj in self.zones():
-            self._service_cleanup_del_res(
-                self.delete_zone,
-                obj,
-                dry_run=dry_run,
-                client_status_queue=client_status_queue,
-                identified_resources=identified_resources,
-                filters=filters,
-                resource_evaluation_fn=resource_evaluation_fn)
-        # Unset all floatingIPs
-        # NOTE: FloatingIPs are not cleaned when filters are set
-        for obj in self.floating_ips():
-            self._service_cleanup_del_res(
-                self.unset_floating_ip,
-                obj,
-                dry_run=dry_run,
-                client_status_queue=client_status_queue,
-                identified_resources=identified_resources,
-                filters=filters,
-                resource_evaluation_fn=resource_evaluation_fn)
+    def _service_cleanup(
+        self,
+        dry_run=True,
+        client_status_queue=False,
+        identified_resources=None,
+        filters=None,
+        resource_evaluation_fn=None,
+        skip_resources=None,
+    ):
+        if not self.should_skip_resource_cleanup("zone", skip_resources):
+            # Delete all zones
+            for obj in self.zones():
+                self._service_cleanup_del_res(
+                    self.delete_zone,
+                    obj,
+                    dry_run=dry_run,
+                    client_status_queue=client_status_queue,
+                    identified_resources=identified_resources,
+                    filters=filters,
+                    resource_evaluation_fn=resource_evaluation_fn,
+                )
+
+        if not self.should_skip_resource_cleanup(
+            "floating_ip", skip_resources
+        ):
+            # Unset all floatingIPs
+            # NOTE: FloatingIPs are not cleaned when filters are set
+            for obj in self.floating_ips():
+                self._service_cleanup_del_res(
+                    self.unset_floating_ip,
+                    obj,
+                    dry_run=dry_run,
+                    client_status_queue=client_status_queue,
+                    identified_resources=identified_resources,
+                    filters=filters,
+                    resource_evaluation_fn=resource_evaluation_fn,
+                )
+
+    # ====== TSIG keys ======
+    def tsigkeys(self, **query):
+        """Retrieve a generator of zones
+
+        :param dict query: Optional query parameters to be sent to limit the
+            resources being returned.
+
+        :returns: A generator of zone
+            :class: `~openstack.dns.v2.tsigkey.TSIGKey` instances.
+        """
+        return self._list(_tsigkey.TSIGKey, **query)
+
+    def create_tsigkey(self, **attrs):
+        """Create a new tsigkey from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.dns.v2.tsigkey.Tsigkey`,
+            comprised of the properties on the Tsigkey class.
+        :returns: The results of zone creation.
+        :rtype: :class:`~openstack.dns.v2.tsigkey.Tsigkey`
+        """
+        return self._create(_tsigkey.TSIGKey, prepend_key=False, **attrs)
+
+    def get_tsigkey(self, tsigkey):
+        """Get a zone
+
+        :param tsigkey: The value can be the ID of a tsigkey
+            or a :class:'~openstack.dns.v2.tsigkey.TSIGKey' instance.
+        :returns: A generator of tsigkey
+            :class:'~openstack.dns.v2.tsigkey.TSIGKey' instances.
+        """
+        return self._get(_tsigkey.TSIGKey, tsigkey)
+
+    def delete_tsigkey(
+        self, tsigkey, ignore_missing=True, delete_shares=False
+    ):
+        """Delete a TSIG key
+
+        :param tsigkey: The value can be the ID of a TSIG key
+            or a :class:`~openstack.dns.v2.tsigkey.TSIGKey` instance.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the TSIG key does not exist.
+            When set to ``True``, no exception will be set when attempting to
+            delete a nonexistent TSIG key.
+
+        :returns: TSIG Key that has been deleted
+        :rtype: :class:`~openstack.dns.v2.tsigkey.TSIGKey`
+        """
+
+        return self._delete(
+            _tsigkey.TSIGKey,
+            tsigkey,
+            ignore_missing=ignore_missing,
+            delete_shares=delete_shares,
+        )
+
+    def find_tsigkey(self, name_or_id, ignore_missing=True):
+        """Find a single tsigkey
+
+        :param name_or_id: The name or ID of a tsigkey
+        :param bool ignore_missing: When set to ``False``
+            :class: `!openstack.exceptions.ResourceNotFound` will be raised
+            when the tsigkey does not exit.
+            Wehn set to ``True``, no exception will be set when attempting
+            to delete a nonexitstent zone.
+
+        :returns::class:`~openstack.dns.v2.tsigkey.TSIGKey`
+        """
+        return self._find(
+            _tsigkey.TSIGKey, name_or_id, ignore_missing=ignore_missing
+        )
