@@ -9,14 +9,14 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-from openstack.common import tag
+
 from openstack import exceptions
 from openstack.network.v2 import _base
 from openstack import resource
 from openstack import utils
 
 
-class Router(_base.NetworkResource, tag.TagMixin):
+class Router(_base.NetworkResource, _base.TagMixinNetwork):
     resource_key = 'router'
     resources_key = 'routers'
     base_path = '/routers'
@@ -40,7 +40,7 @@ class Router(_base.NetworkResource, tag.TagMixin):
         is_admin_state_up='admin_state_up',
         is_distributed='distributed',
         is_ha='ha',
-        **tag.TagMixin._tag_query_parameters,
+        **_base.TagMixinNetwork._tag_query_parameters,
     )
 
     # Properties
