@@ -12,11 +12,13 @@
 
 from calendar import timegm
 import collections
+import functools
 from hashlib import sha1
 import hmac
 import json
 import os
 import time
+import typing as ty
 from urllib import parse
 
 from openstack import _log
@@ -35,6 +37,10 @@ EXPIRES_ISO8601_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
 SHORT_EXPIRES_ISO8601_FORMAT = '%Y-%m-%d'
 
 
+def _get_expiration(expiration):
+    return int(time.time() + expiration)
+
+
 class Proxy(proxy.Proxy):
     _resource_registry = {
         "account": _account.Account,
@@ -51,6 +57,7 @@ class Proxy(proxy.Proxy):
 
     log = _log.setup_logging('openstack')
 
+    @functools.lru_cache(maxsize=256)
     def _extract_name(self, url, service_type=None, project_id=None):
         url_path = parse.urlparse(url).path.strip()
         # Remove / from the beginning to keep the list indexes of interesting
@@ -152,7 +159,7 @@ class Proxy(proxy.Proxy):
         :param container: The value can be either the name of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the container does not exist. When set to ``True``, no exception
             will be set when attempting to delete a nonexistent server.
 
@@ -169,7 +176,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.object_store.v1.container.Container` instance.
 
         :returns: One :class:`~openstack.object_store.v1.container.Container`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._head(_container.Container, container)
@@ -274,7 +281,7 @@ class Proxy(proxy.Proxy):
 
         :returns: Instance of the
             :class:`~openstack.object_store.v1.obj.Object` objects.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         container_name = self._get_container_name(obj=obj, container=container)
@@ -318,7 +325,7 @@ class Proxy(proxy.Proxy):
         :param container: The value can be the name of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
 
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         container_name = self._get_container_name(obj=obj, container=container)
@@ -335,7 +342,7 @@ class Proxy(proxy.Proxy):
         :param container: The value can be the name of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
 
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         :returns: An iterator that iterates over chunk_size bytes
         """
@@ -390,7 +397,8 @@ class Proxy(proxy.Proxy):
         :param metadata: This dict will get changed into headers that set
             metadata of the object
 
-        :raises: ``OpenStackCloudException`` on operation error.
+        :raises: ``:class:`~openstack.exceptions.SDKException``` on operation
+            error.
         """
         if data is not None and filename:
             raise ValueError(
@@ -422,9 +430,7 @@ class Proxy(proxy.Proxy):
             metadata[self._connection._OBJECT_SHA256_KEY] = sha256
 
         container_name = self._get_container_name(container=container)
-        endpoint = '{container}/{name}'.format(
-            container=container_name, name=name
-        )
+        endpoint = f'{container_name}/{name}'
 
         if data is not None:
             self.log.debug(
@@ -485,7 +491,7 @@ class Proxy(proxy.Proxy):
         :param container: The value can be the ID of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the object does not exist.  When set to ``True``, no exception will
             be set when attempting to delete a nonexistent server.
 
@@ -509,7 +515,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.object_store.v1.container.Container` instance.
 
         :returns: One :class:`~openstack.object_store.v1.obj.Object`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         container_name = self._get_container_name(obj, container)
@@ -574,9 +580,7 @@ class Proxy(proxy.Proxy):
             metadata = self.get_object_metadata(name, container).metadata
         except exceptions.NotFoundException:
             self._connection.log.debug(
-                "swift stale check, no object: {container}/{name}".format(
-                    container=container, name=name
-                )
+                f"swift stale check, no object: {container}/{name}"
             )
             return True
 
@@ -600,7 +604,7 @@ class Proxy(proxy.Proxy):
         if not up_to_date:
             self._connection.log.debug(
                 "swift checksum mismatch: "
-                " %(filename)s!=%(container)s/%(name)s",
+                "%(filename)s!=%(container)s/%(name)s",
                 {'filename': filename, 'container': container, 'name': name},
             )
             return True
@@ -646,7 +650,7 @@ class Proxy(proxy.Proxy):
                     # While Object Storage usually expects the name to be
                     # urlencoded in most requests, the SLO manifest requires
                     # plain object names instead.
-                    path='/{name}'.format(name=parse.unquote(name)),
+                    path=f'/{parse.unquote(name)}',
                     size_bytes=segment.length,
                 )
             )
@@ -750,9 +754,7 @@ class Proxy(proxy.Proxy):
                 offset,
                 segment_size if segment_size < remaining else remaining,
             )
-            name = '{endpoint}/{index:0>6}'.format(
-                endpoint=endpoint, index=index
-            )
+            name = f'{endpoint}/{index:0>6}'
             segments[name] = segment
         return segments
 
@@ -764,15 +766,17 @@ class Proxy(proxy.Proxy):
         try:
             # caps = self.get_object_capabilities()
             caps = self.get_info()
-        except exceptions.SDKException as e:
-            if e.response.status_code in (404, 412):
-                server_max_file_size = DEFAULT_MAX_FILE_SIZE
-                self._connection.log.info(
-                    "Swift capabilities not supported. "
-                    "Using default max file size."
-                )
-            else:
-                raise
+        except (
+            exceptions.NotFoundException,
+            exceptions.PreconditionFailedException,
+        ):
+            server_max_file_size = DEFAULT_MAX_FILE_SIZE
+            self._connection.log.info(
+                "Swift capabilities not supported. "
+                "Using default max file size."
+            )
+        except exceptions.SDKException:
+            raise
         else:
             server_max_file_size = caps.swift.get('max_file_size', 0)
             min_segment_size = caps.slo.get('min_segment_size', 0)
@@ -800,7 +804,7 @@ class Proxy(proxy.Proxy):
                 continue
             name = self._object_name_from_url(result.url)
             for entry in manifest:
-                if entry['path'] == '/{name}'.format(name=parse.unquote(name)):
+                if entry['path'] == f'/{parse.unquote(name)}':
                     entry['etag'] = result.headers['Etag']
 
     def get_info(self):
@@ -868,8 +872,8 @@ class Proxy(proxy.Proxy):
             temp_url_key = self.get_temp_url_key(container)
         if not temp_url_key:
             raise exceptions.SDKException(
-                'temp_url_key was not given, nor was a temporary url key'
-                ' found for the account or the container.'
+                'temp_url_key was not given, nor was a temporary url key '
+                'found for the account or the container.'
             )
         return temp_url_key
 
@@ -913,7 +917,7 @@ class Proxy(proxy.Proxy):
             raise exceptions.SDKException(
                 'Please use a positive <timeout> value.'
             )
-        expires = int(time.time() + int(timeout))
+        expires = _get_expiration(timeout)
 
         temp_url_key = self._check_temp_url_key(
             container=container, temp_url_key=temp_url_key
@@ -923,15 +927,8 @@ class Proxy(proxy.Proxy):
         endpoint = parse.urlparse(self.get_endpoint())
         path = '/'.join([endpoint.path, res.name, object_prefix])
 
-        data = '%s\n%s\n%s\n%s\n%s' % (
-            path,
-            redirect_url,
-            max_file_size,
-            max_upload_count,
-            expires,
-        )
-        data = data.encode('utf8')
-        sig = hmac.new(temp_url_key, data, sha1).hexdigest()
+        data = f'{path}\n{redirect_url}\n{max_file_size}\n{max_upload_count}\n{expires}'
+        sig = hmac.new(temp_url_key, data.encode(), sha1).hexdigest()
 
         return (expires, sig)
 
@@ -987,18 +984,17 @@ class Proxy(proxy.Proxy):
                     try:
                         t = time.strptime(seconds, f)
                     except ValueError:
-                        t = None
+                        continue
+
+                    if f == EXPIRES_ISO8601_FORMAT:
+                        timestamp = timegm(t)
                     else:
-                        if f == EXPIRES_ISO8601_FORMAT:
-                            timestamp = timegm(t)
-                        else:
-                            # Use local time if UTC designator is missing.
-                            timestamp = int(time.mktime(t))
+                        # Use local time if UTC designator is missing.
+                        timestamp = int(time.mktime(t))
 
-                        absolute = True
-                        break
-
-                if t is None:
+                    absolute = True
+                    break
+                else:
                     raise ValueError()
             else:
                 if not timestamp.is_integer():
@@ -1041,8 +1037,9 @@ class Proxy(proxy.Proxy):
                 method.upper(),
             )
 
+        expiration: ty.Union[float, int]
         if not absolute:
-            expiration = int(time.time() + timestamp)
+            expiration = _get_expiration(timestamp)
         else:
             expiration = timestamp
 
@@ -1058,9 +1055,9 @@ class Proxy(proxy.Proxy):
                     ip_range = ip_range.decode('utf-8')
                 except UnicodeDecodeError:
                     raise ValueError('ip_range must be representable as UTF-8')
-            hmac_parts.insert(0, "ip=%s" % ip_range)
+            hmac_parts.insert(0, f"ip={ip_range}")
 
-        hmac_body = u'\n'.join(hmac_parts)
+        hmac_body = '\n'.join(hmac_parts)
 
         temp_url_key = self._check_temp_url_key(temp_url_key=temp_url_key)
 
@@ -1069,19 +1066,19 @@ class Proxy(proxy.Proxy):
         ).hexdigest()
 
         if iso8601:
-            expiration = time.strftime(
+            exp = time.strftime(
                 EXPIRES_ISO8601_FORMAT, time.gmtime(expiration)
             )
+        else:
+            exp = str(expiration)
 
-        temp_url = u'{path}?temp_url_sig={sig}&temp_url_expires={exp}'.format(
-            path=path_for_body, sig=sig, exp=expiration
-        )
+        temp_url = f'{path_for_body}?temp_url_sig={sig}&temp_url_expires={exp}'
 
         if ip_range:
-            temp_url += u'&temp_url_ip_range={}'.format(ip_range)
+            temp_url += f'&temp_url_ip_range={ip_range}'
 
         if prefix:
-            temp_url += u'&temp_url_prefix={}'.format(parts[4])
+            temp_url += f'&temp_url_prefix={parts[4]}'
         # Have return type match path from caller
         if isinstance(path, bytes):
             return temp_url.encode('utf-8')
@@ -1138,17 +1135,18 @@ class Proxy(proxy.Proxy):
             return
 
         is_bulk_delete_supported = False
-        bulk_delete_max_per_request = None
+        bulk_delete_max_per_request = 1
         try:
             caps = self.get_info()
         except exceptions.SDKException:
             pass
         else:
-            bulk_delete = caps.swift.get("bulk_delete", {})
-            is_bulk_delete_supported = bulk_delete is not None
-            bulk_delete_max_per_request = bulk_delete.get(
-                "max_deletes_per_request", 100
-            )
+            bulk_delete = caps.get("bulk_delete")
+            if bulk_delete is not None:
+                is_bulk_delete_supported = True
+                bulk_delete_max_per_request = bulk_delete.get(
+                    "max_deletes_per_request", 10000
+                )
 
         elements = []
         for cont in self.containers():
@@ -1165,18 +1163,20 @@ class Proxy(proxy.Proxy):
                     resource_evaluation_fn=resource_evaluation_fn,
                 )
                 if need_delete:
-                    if not is_bulk_delete_supported and not dry_run:
-                        self.delete_object(obj, cont)
-                    else:
+                    if dry_run:
+                        continue
+                    elif is_bulk_delete_supported:
                         elements.append(f"{cont.name}/{obj.name}")
                         if len(elements) >= bulk_delete_max_per_request:
-                            self._bulk_delete(elements, dry_run=dry_run)
+                            self._bulk_delete(elements)
                             elements.clear()
+                    else:
+                        self.delete_object(obj, cont)
                 else:
                     objects_remaining = True
 
             if len(elements) > 0:
-                self._bulk_delete(elements, dry_run=dry_run)
+                self._bulk_delete(elements)
                 elements.clear()
 
             # Eventually delete container itself
@@ -1191,14 +1191,13 @@ class Proxy(proxy.Proxy):
                     resource_evaluation_fn=resource_evaluation_fn,
                 )
 
-    def _bulk_delete(self, elements, dry_run=False):
+    def _bulk_delete(self, elements):
         data = "\n".join([parse.quote(x) for x in elements])
-        if not dry_run:
-            self.delete(
-                "?bulk-delete",
-                data=data,
-                headers={
-                    'Content-Type': 'text/plain',
-                    'Accept': 'application/json',
-                },
-            )
+        self.delete(
+            "?bulk-delete",
+            data=data,
+            headers={
+                'Content-Type': 'text/plain',
+                'Accept': 'application/json',
+            },
+        )
