@@ -9,13 +9,15 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
+from openstack import exceptions
 from openstack import proxy
 from openstack import resource
 from openstack.shared_file_system.v2 import (
     availability_zone as _availability_zone,
 )
 from openstack.shared_file_system.v2 import limit as _limit
+from openstack.shared_file_system.v2 import quota_class_set as _quota_class_set
+from openstack.shared_file_system.v2 import resource_locks as _resource_locks
 from openstack.shared_file_system.v2 import share as _share
 from openstack.shared_file_system.v2 import share_group as _share_group
 from openstack.shared_file_system.v2 import (
@@ -56,6 +58,8 @@ class Proxy(proxy.Proxy):
         "share_access_rule": _share_access_rule.ShareAccessRule,
         "share_group": _share_group.ShareGroup,
         "share_group_snapshot": _share_group_snapshot.ShareGroupSnapshot,
+        "resource_locks": _resource_locks.ResourceLock,
+        "quota_class_set": _quota_class_set.QuotaClassSet,
     }
 
     def availability_zones(self):
@@ -120,7 +124,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a share.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -187,6 +191,42 @@ class Proxy(proxy.Proxy):
         res = self._get(_share.Share, share_id)
         res.revert_to_snapshot(self, snapshot_id)
 
+    def manage_share(self, protocol, export_path, service_host, **params):
+        """Manage a share.
+
+        :param str protocol: The shared file systems protocol of this share.
+        :param str export_path: The export path formatted according to the
+            protocol.
+        :param str service_host: The manage-share service host.
+        :param kwargs params: Optional parameters to be sent. Available
+            parameters include:
+            * name: The user defined name of the resource.
+            * share_type: The name or ID of the share type to be used to create
+            the resource.
+            * driver_options: A set of one or more key and value pairs, as a
+            dictionary of strings, that describe driver options.
+            * is_public: The level of visibility for the share.
+            * description: The user defiend description of the resource.
+            * share_server_id: The UUID of the share server.
+
+        :returns: The share that was managed.
+        """
+
+        share = _share.Share()
+        return share.manage(
+            self, protocol, export_path, service_host, **params
+        )
+
+    def unmanage_share(self, share_id):
+        """Unmanage the share with the given share ID.
+
+        :param share_id: The ID of the share to unmanage.
+        :returns: ``None``
+        """
+
+        share_to_unmanage = self._get(_share.Share, share_id)
+        share_to_unmanage.unmanage(self)
+
     def resize_share(
         self, share_id, new_size, no_shrink=False, no_extend=False, force=False
     ):
@@ -272,7 +312,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a share group.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -318,7 +358,13 @@ class Proxy(proxy.Proxy):
         )
 
     def wait_for_status(
-        self, res, status='active', failures=None, interval=2, wait=120
+        self,
+        res,
+        status='active',
+        failures=None,
+        interval=2,
+        wait=120,
+        status_attr_name='status',
     ):
         """Wait for a resource to be in a particular status.
         :param res: The resource to wait on to reach the specified status.
@@ -331,6 +377,8 @@ class Proxy(proxy.Proxy):
             checks. Default to 2.
         :param wait: Maximum number of seconds to wait before the change.
             Default to 120.
+        :param status_attr_name: name of the attribute to reach the desired
+            status.
         :returns: The resource is returned on success.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
             to the desired status failed to occur in specified seconds.
@@ -341,7 +389,13 @@ class Proxy(proxy.Proxy):
         """
         failures = [] if failures is None else failures
         return resource.wait_for_status(
-            self, res, status, failures, interval, wait
+            self,
+            res,
+            status,
+            failures,
+            interval,
+            wait,
+            attribute=status_attr_name,
         )
 
     def storage_pools(self, details=True, **query):
@@ -542,7 +596,7 @@ class Proxy(proxy.Proxy):
         return self._create(
             _share_network_subnet.ShareNetworkSubnet,
             **attrs,
-            share_network_id=share_network_id
+            share_network_id=share_network_id,
         )
 
     def delete_share_network_subnet(
@@ -602,7 +656,7 @@ class Proxy(proxy.Proxy):
         return self._list(
             _share_snapshot_instance.ShareSnapshotInstance,
             base_path=base_path,
-            **query
+            **query,
         )
 
     def get_share_snapshot_instance(self, snapshot_instance_id):
@@ -805,22 +859,35 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.shared_file_system.v2.
             share_access_rules.ShareAccessRules`
         """
-        base_path = "/shares/%s/action" % (share_id,)
+        base_path = f"/shares/{share_id}/action"
         return self._create(
             _share_access_rule.ShareAccessRule, base_path=base_path, **attrs
         )
 
-    def delete_access_rule(self, access_id, share_id, ignore_missing=True):
+    def delete_access_rule(
+        self, access_id, share_id, ignore_missing=True, *, unrestrict=False
+    ):
         """Deletes an access rule
 
         :param access_id: The id of the access rule to get
         :param share_id: The ID of the share
+        :param unrestrict: If Manila must attempt removing locks while deleting
 
         :rtype: ``requests.models.Response`` HTTP response from internal
             requests client
         """
-        res = self._get_resource(_share_access_rule.ShareAccessRule, access_id)
-        return res.delete(self, share_id, ignore_missing=ignore_missing)
+        res = self._get_resource(
+            _share_access_rule.ShareAccessRule, access_id, share_id=share_id
+        )
+        try:
+            return res.delete(
+                self,
+                unrestrict=unrestrict,
+            )
+        except exceptions.NotFoundException:
+            if ignore_missing:
+                return None
+            raise
 
     def share_group_snapshots(self, details=True, **query):
         """Lists all share group snapshots.
@@ -851,7 +918,7 @@ class Proxy(proxy.Proxy):
         return self._list(
             _share_group_snapshot.ShareGroupSnapshot,
             base_path=base_path,
-            **query
+            **query,
         )
 
     def share_group_snapshot_members(self, group_snapshot_id):
@@ -896,7 +963,7 @@ class Proxy(proxy.Proxy):
         return self._create(
             _share_group_snapshot.ShareGroupSnapshot,
             share_group_id=share_group_id,
-            **attrs
+            **attrs,
         )
 
     def reset_share_group_snapshot_status(self, group_snapshot_id, status):
@@ -925,7 +992,7 @@ class Proxy(proxy.Proxy):
         return self._update(
             _share_group_snapshot.ShareGroupSnapshot,
             group_snapshot_id,
-            **attrs
+            **attrs,
         )
 
     def delete_share_group_snapshot(
@@ -941,3 +1008,232 @@ class Proxy(proxy.Proxy):
             group_snapshot_id,
             ignore_missing=ignore_missing,
         )
+
+    # ========= Share Metadata ==========
+    def get_share_metadata(self, share_id):
+        """Lists all metadata for a share.
+
+        :param share_id: The ID of the share
+
+        :returns: A :class:`~openstack.shared_file_system.v2.share.Share`
+            with the share's metadata.
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share.Share`
+        """
+        share = self._get_resource(_share.Share, share_id)
+        return share.fetch_metadata(self)
+
+    def get_share_metadata_item(self, share_id, key):
+        """Retrieves a specific metadata item from a share by its key.
+
+        :param share_id: The ID of the share
+        :param key: The key of the share metadata
+
+        :returns: A :class:`~openstack.shared_file_system.v2.share.Share`
+            with the share's metadata.
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share.Share`
+        """
+        share = self._get_resource(_share.Share, share_id)
+        return share.get_metadata_item(self, key)
+
+    def create_share_metadata(self, share_id, **metadata):
+        """Creates share metadata as key-value pairs.
+
+        :param share_id: The ID of the share
+        :param metadata: The metadata to be created
+
+        :returns: A :class:`~openstack.shared_file_system.v2.share.Share`
+            with the share's metadata.
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share.Share`
+        """
+        share = self._get_resource(_share.Share, share_id)
+        return share.set_metadata(self, metadata=metadata)
+
+    def update_share_metadata(self, share_id, metadata, replace=False):
+        """Updates metadata of given share.
+
+        :param share_id: The ID of the share
+        :param metadata: The metadata to be created
+        :param replace: Boolean for whether the preexisting metadata
+            should be replaced
+
+        :returns: A :class:`~openstack.shared_file_system.v2.share.Share`
+            with the share's updated metadata.
+        :rtype:
+            :class:`~openstack.shared_file_system.v2.share.Share`
+        """
+        share = self._get_resource(_share.Share, share_id)
+        return share.set_metadata(self, metadata=metadata, replace=replace)
+
+    def delete_share_metadata(self, share_id, keys, ignore_missing=True):
+        """Deletes a single metadata item on a share, idetified by its key.
+
+        :param share_id: The ID of the share
+        :param keys: The list of share metadata keys to be deleted
+        :param ignore_missing: Boolean indicating if missing keys should be ignored.
+
+        :returns: None
+        :rtype: None
+        """
+        share = self._get_resource(_share.Share, share_id)
+        keys_failed_to_delete = []
+        for key in keys:
+            try:
+                share.delete_metadata_item(self, key)
+            except exceptions.NotFoundException:
+                if not ignore_missing:
+                    self._connection.log.info("Key %s not found.", key)
+                    keys_failed_to_delete.append(key)
+            except exceptions.ForbiddenException:
+                self._connection.log.info("Key %s cannot be deleted.", key)
+                keys_failed_to_delete.append(key)
+            except exceptions.SDKException:
+                self._connection.log.info("Failed to delete key %s.", key)
+                keys_failed_to_delete.append(key)
+        if keys_failed_to_delete:
+            raise exceptions.SDKException(
+                f"Some keys failed to be deleted {keys_failed_to_delete}"
+            )
+
+    def resource_locks(self, **query):
+        """Lists all resource locks.
+
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resource locks being returned.  Available parameters include:
+
+            * project_id: The project ID of the user that the lock is
+                created for.
+            * user_id: The ID of a user to filter resource locks by.
+            * all_projects: list locks from all projects (Admin Only)
+            * resource_id: The ID of the resource that the locks pertain to
+                filter resource locks by.
+            * resource_action: The action prevented by the filtered resource
+                locks.
+            * resource_type: The type of the resource that the locks pertain
+                to filter resource locks by.
+            * lock_context: The lock creator’s context to filter locks by.
+            * lock_reason: The lock reason that can be used to filter resource
+                locks. (Inexact search is also available with lock_reason~)
+            * created_since: Search for the list of resources that were created
+                after the specified date. The date is in ‘yyyy-mm-dd’ format.
+            * created_before: Search for the list of resources that were
+                created prior to the specified date. The date is in
+                ‘yyyy-mm-dd’ format.
+            * limit: The maximum number of resource locks to return.
+            * offset: The offset to define start point of resource lock
+                listing.
+            * sort_key: The key to sort a list of shares.
+            * sort_dir: The direction to sort a list of shares
+            * with_count: Whether to show count in API response or not,
+                default is False. This query parameter is useful with
+                pagination.
+
+        :returns: A generator of manila resource locks
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            resource_locks.ResourceLock`
+        """
+
+        if query.get('resource_type'):
+            # The _create method has a parameter named resource_type, which
+            # refers to the type of resource to be created, so we need to avoid
+            # a conflict of parameters we are sending to the method.
+            query['__conflicting_attrs'] = {
+                'resource_type': query.get('resource_type')
+            }
+            query.pop('resource_type')
+        return self._list(_resource_locks.ResourceLock, **query)
+
+    def get_resource_lock(self, resource_lock):
+        """Show details of a resource lock.
+
+        :param resource_lock: The ID of a resource lock or a
+            :class:`~openstack.shared_file_system.v2.
+            resource_locks.ResourceLock` instance.
+        :returns: Details of the identified resource lock.
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            resource_locks.ResourceLock`
+        """
+        return self._get(_resource_locks.ResourceLock, resource_lock)
+
+    def update_resource_lock(self, resource_lock, **attrs):
+        """Updates details of a single resource lock.
+
+        :param resource_lock: The ID of a resource lock or a
+            :class:`~openstack.shared_file_system.v2.
+            resource_locks.ResourceLock` instance.
+        :param dict attrs: The attributes to update on the resource lock
+        :returns: the updated resource lock
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            resource_locks.ResourceLock`
+        """
+        return self._update(
+            _resource_locks.ResourceLock, resource_lock, **attrs
+        )
+
+    def delete_resource_lock(self, resource_lock, ignore_missing=True):
+        """Deletes a single resource lock
+
+        :param resource_lock: The ID of a resource lock or a
+            :class:`~openstack.shared_file_system.v2.
+            resource_locks.ResourceLock` instance.
+        :returns: Result of the ``delete``
+        :rtype: ``None``
+        """
+        return self._delete(
+            _resource_locks.ResourceLock,
+            resource_lock,
+            ignore_missing=ignore_missing,
+        )
+
+    def create_resource_lock(self, **attrs):
+        """Locks a resource.
+
+        :param dict attrs: Attributes which will be used to create
+            a :class:`~openstack.shared_file_system.v2.
+            resource_locks.ResourceLock`, comprised of the properties
+            on the ResourceLock class. Available parameters include:
+
+            * ``resource_id``: ID of the resource to be locked.
+            * ``resource_type``: type of the resource (share, access_rule).
+            * ``resource_action``: action to be locked (delete, show).
+            * ``lock_reason``: reason why you're locking the resource
+                (Optional).
+        :returns: Details of the lock
+        :rtype: :class:`~openstack.shared_file_system.v2.
+            resource_locks.ResourceLock`
+        """
+
+        if attrs.get('resource_type'):
+            # The _create method has a parameter named resource_type, which
+            # refers to the type of resource to be created, so we need to avoid
+            # a conflict of parameters we are sending to the method.
+            attrs['__conflicting_attrs'] = {
+                'resource_type': attrs.get('resource_type')
+            }
+            attrs.pop('resource_type')
+        return self._create(_resource_locks.ResourceLock, **attrs)
+
+    def get_quota_class_set(self, quota_class_name):
+        """Get quota class set.
+
+        :param quota_class_name: The name of the quota class
+        :returns: A :class:`~openstack.shared_file_system.v2
+            .quota_class_set.QuotaClassSet`
+        """
+        return self._get(_quota_class_set.QuotaClassSet, quota_class_name)
+
+    def update_quota_class_set(self, quota_class_name, **attrs):
+        """Update quota class set.
+
+        :param quota_class_name: The name of the quota class
+        :param attrs: The attributes to update on the quota class set
+        :returns: the updated quota class set
+        :rtype: :class:`~openstack.shared_file_system.v2
+            .quota_class_set.QuotaClassSet`
+        """
+
+        return self._update(
+            _quota_class_set.QuotaClassSet, quota_class_name, **attrs
+        )
