@@ -57,7 +57,7 @@ _GroupData = collections.namedtuple(
 
 _DomainData = collections.namedtuple(
     'DomainData',
-    'domain_id, domain_name, description, json_response, ' 'json_request',
+    'domain_id, domain_name, description, json_response, json_request',
 )
 
 
@@ -88,7 +88,7 @@ class TestCase(base.TestCase):
     def setUp(self, cloud_config_fixture='clouds.yaml'):
         """Run before each test method to initialize test environment."""
 
-        super(TestCase, self).setUp()
+        super().setUp()
 
         # Sleeps are for real testing, but unit tests shouldn't need them
         realsleep = time.sleep
@@ -105,9 +105,13 @@ class TestCase(base.TestCase):
         )
 
         # Isolate openstack.config from test environment
+        self.os_cloud_fixture = self.useFixture(
+            fixtures.EnvironmentVariable('OS_CLOUD'),
+        )
         config = tempfile.NamedTemporaryFile(delete=False)
-        cloud_path = '%s/clouds/%s' % (
+        cloud_path = os.path.join(
             self.fixtures_directory,
+            'clouds',
             cloud_config_fixture,
         )
         with open(cloud_path, 'rb') as f:
@@ -199,8 +203,8 @@ class TestCase(base.TestCase):
         if append:
             to_join.extend([urllib.parse.quote(i) for i in append])
         if qs_elements is not None:
-            qs = '?%s' % '&'.join(qs_elements)
-        return '%(uri)s%(qs)s' % {'uri': '/'.join(to_join), 'qs': qs}
+            qs = '?{}'.format('&'.join(qs_elements))
+        return '{uri}{qs}'.format(uri='/'.join(to_join), qs=qs)
 
     def mock_for_keystone_projects(
         self,
@@ -219,7 +223,7 @@ class TestCase(base.TestCase):
             assert not (project or project_list)
         else:
             raise Exception(
-                'Must specify a project, project_list, ' 'or project_count'
+                'Must specify a project, project_list, or project_count'
             )
         assert list_get or id_get
 
@@ -358,7 +362,7 @@ class TestCase(base.TestCase):
         self.assertIs(
             0,
             len(kwargs),
-            message='extra key-word args received ' 'on _get_user_data',
+            message='extra key-word args received on _get_user_data',
         )
 
         return _UserData(
@@ -477,7 +481,7 @@ class TestCase(base.TestCase):
                 dict(
                     method='GET',
                     uri='https://identity.example.com/',
-                    text=open(self.discovery_json, 'r').read(),
+                    text=open(self.discovery_json).read(),
                 ),
                 dict(
                     method='POST',
@@ -528,7 +532,7 @@ class TestCase(base.TestCase):
         )
 
     def get_keystone_discovery(self):
-        with open(self.discovery_json, 'r') as discovery_file:
+        with open(self.discovery_json) as discovery_file:
             return dict(
                 method='GET',
                 uri='https://identity.example.com/',
@@ -587,7 +591,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri=block_storage_discovery_url,
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_glance_discovery_mock_dict(
@@ -602,7 +606,7 @@ class TestCase(base.TestCase):
             method='GET',
             uri=image_discovery_url,
             status_code=300,
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_nova_discovery_mock_dict(
@@ -616,7 +620,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri=compute_discovery_url,
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_placement_discovery_mock_dict(
@@ -628,7 +632,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://placement.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_designate_discovery_mock_dict(self):
@@ -636,7 +640,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://dns.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_ironic_discovery_mock_dict(self):
@@ -646,7 +650,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://baremetal.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_senlin_discovery_mock_dict(self):
@@ -656,7 +660,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://clustering.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def use_compute_discovery(
@@ -679,7 +683,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://accelerator.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_manila_discovery_mock_dict(self):
@@ -689,7 +693,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://shared-file-system.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def use_glance(
@@ -807,23 +811,19 @@ class TestCase(base.TestCase):
             # NOTE(notmorgan): make sure the delimiter is non-url-safe, in this
             # case "|" is used so that the split can be a bit easier on
             # maintainers of this code.
-            key = '{method}|{uri}|{params}'.format(
-                method=method, uri=uri, params=kw_params
-            )
+            key = f'{method}|{uri}|{kw_params}'
             validate = to_mock.pop('validate', {})
-            valid_keys = set(['json', 'headers', 'params', 'data'])
+            valid_keys = {'json', 'headers', 'params', 'data'}
             invalid_keys = set(validate.keys()) - valid_keys
             if invalid_keys:
                 raise TypeError(
-                    "Invalid values passed to validate: {keys}".format(
-                        keys=invalid_keys
-                    )
+                    f"Invalid values passed to validate: {invalid_keys}"
                 )
             headers = structures.CaseInsensitiveDict(
                 to_mock.pop('headers', {})
             )
             if 'content-type' not in headers:
-                headers[u'content-type'] = 'application/json'
+                headers['content-type'] = 'application/json'
 
             if 'exc' not in to_mock:
                 to_mock['headers'] = headers
@@ -837,11 +837,10 @@ class TestCase(base.TestCase):
                     'PROGRAMMING ERROR: key-word-params '
                     'should be part of the uri_key and cannot change, '
                     'it will affect the matcher in requests_mock. '
-                    '%(old)r != %(new)r'
-                    % {
-                        'old': self._uri_registry[key]['kw_params'],
-                        'new': kw_params,
-                    }
+                    '{old!r} != {new!r}'.format(
+                        old=self._uri_registry[key]['kw_params'],
+                        new=kw_params,
+                    )
                 )
             self._uri_registry[key]['response_list'].append(to_mock)
 
@@ -851,7 +850,7 @@ class TestCase(base.TestCase):
                 mock_method,
                 mock_uri,
                 params['response_list'],
-                **params['kw_params']
+                **params['kw_params'],
             )
 
     def assert_no_calls(self):
@@ -896,9 +895,7 @@ class TestCase(base.TestCase):
                         'call': '{method} {url}'.format(
                             method=call['method'], url=call['url']
                         ),
-                        'history': '{method} {url}'.format(
-                            method=history.method, url=history.url
-                        ),
+                        'history': f'{history.method} {history.url}',
                     }
                 ),
             )
@@ -906,7 +903,7 @@ class TestCase(base.TestCase):
                 self.assertEqual(
                     call['json'],
                     history.json(),
-                    'json content mismatch in call {index}'.format(index=x),
+                    f'json content mismatch in call {x}',
                 )
             # headers in a call isn't exhaustive - it's checking to make sure
             # a specific header or headers are there, not that they are the
@@ -916,11 +913,16 @@ class TestCase(base.TestCase):
                     self.assertEqual(
                         value,
                         history.headers[key],
-                        'header mismatch in call {index}'.format(index=x),
+                        f'header mismatch in call {x}',
                     )
         if do_count:
             self.assertEqual(
-                len(self.calls), len(self.adapter.request_history)
+                len(self.calls),
+                len(self.adapter.request_history),
+                "Expected:\n{}'\nGot:\n{}".format(
+                    '\n'.join([c['url'] for c in self.calls]),
+                    '\n'.join([h.url for h in self.adapter.request_history]),
+                ),
             )
 
     def assertResourceEqual(self, actual, expected, resource_type):
@@ -955,7 +957,7 @@ class TestCase(base.TestCase):
 
 class IronicTestCase(TestCase):
     def setUp(self):
-        super(IronicTestCase, self).setUp()
+        super().setUp()
         self.use_ironic()
         self.uuid = str(uuid.uuid4())
         self.name = self.getUniqueString('name')
@@ -964,4 +966,4 @@ class IronicTestCase(TestCase):
         kwargs.setdefault('service_type', 'baremetal')
         kwargs.setdefault('interface', 'public')
         kwargs.setdefault('base_url_append', 'v1')
-        return super(IronicTestCase, self).get_mock_url(**kwargs)
+        return super().get_mock_url(**kwargs)
