@@ -13,25 +13,35 @@
 import uuid
 
 from openstack.identity.v3 import _proxy
+from openstack.identity.v3 import access_rule
 from openstack.identity.v3 import credential
 from openstack.identity.v3 import domain
+from openstack.identity.v3 import domain_config
 from openstack.identity.v3 import endpoint
 from openstack.identity.v3 import group
 from openstack.identity.v3 import policy
 from openstack.identity.v3 import project
 from openstack.identity.v3 import region
 from openstack.identity.v3 import role
+from openstack.identity.v3 import role_domain_group_assignment
+from openstack.identity.v3 import role_domain_user_assignment
+from openstack.identity.v3 import role_project_group_assignment
+from openstack.identity.v3 import role_project_user_assignment
+from openstack.identity.v3 import role_system_group_assignment
+from openstack.identity.v3 import role_system_user_assignment
 from openstack.identity.v3 import service
+from openstack.identity.v3 import service_provider
 from openstack.identity.v3 import trust
 from openstack.identity.v3 import user
 from openstack.tests.unit import test_proxy_base
 
 USER_ID = 'user-id-' + uuid.uuid4().hex
+ENDPOINT_ID = 'user-id-' + uuid.uuid4().hex
 
 
 class TestIdentityProxyBase(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestIdentityProxyBase, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
 
@@ -85,6 +95,75 @@ class TestIdentityProxyDomain(TestIdentityProxyBase):
         self.verify_update(self.proxy.update_domain, domain.Domain)
 
 
+class TestIdentityProxyDomainConfig(TestIdentityProxyBase):
+    def test_domain_config_create_attrs(self):
+        self.verify_create(
+            self.proxy.create_domain_config,
+            domain_config.DomainConfig,
+            method_args=['domain_id'],
+            method_kwargs={},
+            expected_args=[],
+            expected_kwargs={
+                'domain_id': 'domain_id',
+            },
+        )
+
+    def test_domain_config_delete(self):
+        self.verify_delete(
+            self.proxy.delete_domain_config,
+            domain_config.DomainConfig,
+            ignore_missing=False,
+            method_args=['domain_id'],
+            method_kwargs={},
+            expected_args=[],
+            expected_kwargs={
+                'domain_id': 'domain_id',
+            },
+        )
+
+    def test_domain_config_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_domain_config,
+            domain_config.DomainConfig,
+            ignore_missing=True,
+            method_args=['domain_id'],
+            method_kwargs={},
+            expected_args=[],
+            expected_kwargs={
+                'domain_id': 'domain_id',
+            },
+        )
+
+    # no find_domain_config
+
+    def test_domain_config_get(self):
+        self.verify_get(
+            self.proxy.get_domain_config,
+            domain_config.DomainConfig,
+            method_args=['domain_id'],
+            method_kwargs={},
+            expected_args=[],
+            expected_kwargs={
+                'domain_id': 'domain_id',
+                'requires_id': False,
+            },
+        )
+
+    # no domain_configs
+
+    def test_domain_config_update(self):
+        self.verify_update(
+            self.proxy.update_domain_config,
+            domain_config.DomainConfig,
+            method_args=['domain_id'],
+            method_kwargs={},
+            expected_args=[],
+            expected_kwargs={
+                'domain_id': 'domain_id',
+            },
+        )
+
+
 class TestIdentityProxyEndpoint(TestIdentityProxyBase):
     def test_endpoint_create_attrs(self):
         self.verify_create(self.proxy.create_endpoint, endpoint.Endpoint)
@@ -224,6 +303,14 @@ class TestIdentityProxyProject(TestIdentityProxyBase):
             expected_kwargs={'user_id': USER_ID},
         )
 
+    def test_endpoint_projects(self):
+        self.verify_list(
+            self.proxy.endpoint_projects,
+            project.EndpointProject,
+            method_kwargs={'endpoint': ENDPOINT_ID},
+            expected_kwargs={'endpoint_id': ENDPOINT_ID},
+        )
+
     def test_project_update(self):
         self.verify_update(self.proxy.update_project, project.Project)
 
@@ -341,6 +428,72 @@ class TestIdentityProxyRole(TestIdentityProxyBase):
 
 
 class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
+    def test_role_assignments_filter__domain_user(self):
+        self.verify_list(
+            self.proxy.role_assignments_filter,
+            role_domain_user_assignment.RoleDomainUserAssignment,
+            method_kwargs={'domain': 'domain', 'user': 'user'},
+            expected_kwargs={
+                'domain_id': 'domain',
+                'user_id': 'user',
+            },
+        )
+
+    def test_role_assignments_filter__domain_group(self):
+        self.verify_list(
+            self.proxy.role_assignments_filter,
+            role_domain_group_assignment.RoleDomainGroupAssignment,
+            method_kwargs={'domain': 'domain', 'group': 'group'},
+            expected_kwargs={
+                'domain_id': 'domain',
+                'group_id': 'group',
+            },
+        )
+
+    def test_role_assignments_filter__project_user(self):
+        self.verify_list(
+            self.proxy.role_assignments_filter,
+            role_project_user_assignment.RoleProjectUserAssignment,
+            method_kwargs={'project': 'project', 'user': 'user'},
+            expected_kwargs={
+                'project_id': 'project',
+                'user_id': 'user',
+            },
+        )
+
+    def test_role_assignments_filter__project_group(self):
+        self.verify_list(
+            self.proxy.role_assignments_filter,
+            role_project_group_assignment.RoleProjectGroupAssignment,
+            method_kwargs={'project': 'project', 'group': 'group'},
+            expected_kwargs={
+                'project_id': 'project',
+                'group_id': 'group',
+            },
+        )
+
+    def test_role_assignments_filter__system_user(self):
+        self.verify_list(
+            self.proxy.role_assignments_filter,
+            role_system_user_assignment.RoleSystemUserAssignment,
+            method_kwargs={'system': 'system', 'user': 'user'},
+            expected_kwargs={
+                'system_id': 'system',
+                'user_id': 'user',
+            },
+        )
+
+    def test_role_assignments_filter__system_group(self):
+        self.verify_list(
+            self.proxy.role_assignments_filter,
+            role_system_group_assignment.RoleSystemGroupAssignment,
+            method_kwargs={'system': 'system', 'group': 'group'},
+            expected_kwargs={
+                'system_id': 'system',
+                'group_id': 'group',
+            },
+        )
+
     def test_assign_domain_role_to_user(self):
         self._verify(
             "openstack.identity.v3.domain.Domain.assign_role_to_user",
@@ -351,6 +504,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -364,6 +518,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -377,6 +532,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -390,6 +546,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -403,6 +560,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -416,6 +574,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -429,6 +588,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -442,6 +602,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -455,6 +616,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(user.User, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -468,6 +630,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -481,6 +644,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -494,6 +658,7 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy,
                 self.proxy._get_resource(group.Group, 'uid'),
                 self.proxy._get_resource(role.Role, 'rid'),
+                False,
             ],
         )
 
@@ -568,3 +733,90 @@ class TestIdentityProxyRoleAssignments(TestIdentityProxyBase):
                 self.proxy._get_resource(role.Role, 'rid'),
             ],
         )
+
+
+class TestAccessRule(TestIdentityProxyBase):
+    def test_access_rule_delete(self):
+        self.verify_delete(
+            self.proxy.delete_access_rule,
+            access_rule.AccessRule,
+            False,
+            method_args=[],
+            method_kwargs={'user': USER_ID, 'access_rule': 'access_rule'},
+            expected_args=['access_rule'],
+            expected_kwargs={'user_id': USER_ID},
+        )
+
+    def test_access_rule_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_access_rule,
+            access_rule.AccessRule,
+            True,
+            method_args=[],
+            method_kwargs={'user': USER_ID, 'access_rule': 'access_rule'},
+            expected_args=['access_rule'],
+            expected_kwargs={'user_id': USER_ID},
+        )
+
+    def test_access_rule_get(self):
+        self.verify_get(
+            self.proxy.get_access_rule,
+            access_rule.AccessRule,
+            method_args=[],
+            method_kwargs={'user': USER_ID, 'access_rule': 'access_rule'},
+            expected_args=['access_rule'],
+            expected_kwargs={'user_id': USER_ID},
+        )
+
+    def test_access_rules(self):
+        self.verify_list(
+            self.proxy.access_rules,
+            access_rule.AccessRule,
+            method_kwargs={'user': USER_ID},
+            expected_kwargs={'user_id': USER_ID},
+        )
+
+
+class TestServiceProvider(TestIdentityProxyBase):
+    def test_service_provider_create(self):
+        self.verify_create(
+            self.proxy.create_service_provider,
+            service_provider.ServiceProvider,
+        )
+
+    def test_service_provider_delete(self):
+        self.verify_delete(
+            self.proxy.delete_service_provider,
+            service_provider.ServiceProvider,
+            False,
+        )
+
+    def test_service_provider_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_service_provider,
+            service_provider.ServiceProvider,
+            True,
+        )
+
+    def test_service_provider_find(self):
+        self.verify_find(
+            self.proxy.find_service_provider, service_provider.ServiceProvider
+        )
+
+    def test_service_provider_get(self):
+        self.verify_get(
+            self.proxy.get_service_provider,
+            service_provider.ServiceProvider,
+        )
+
+    def test_service_providers(self):
+        self.verify_list(
+            self.proxy.service_providers,
+            service_provider.ServiceProvider,
+        )
+
+    def test_service_provider_update(self):
+        self.verify_update(
+            self.proxy.update_service_provider,
+            service_provider.ServiceProvider,
+        )
