@@ -41,13 +41,11 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
     kwargs_to_path_args = False
 
     def setUp(self):
-        super(TestObjectStoreProxy, self).setUp()
+        super().setUp()
         self.proxy = self.cloud.object_store
         self.container = self.getUniqueString()
         self.endpoint = self.cloud.object_store.get_endpoint() + '/'
-        self.container_endpoint = '{endpoint}{container}'.format(
-            endpoint=self.endpoint, container=self.container
-        )
+        self.container_endpoint = f'{self.endpoint}{self.container}'
 
     def test_account_metadata_get(self):
         self.verify_head(
@@ -132,13 +130,13 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
 
     def test_object_get(self):
         with requests_mock.Mocker() as m:
-            m.get("%scontainer/object" % self.endpoint, text="data")
+            m.get(f"{self.endpoint}container/object", text="data")
             res = self.proxy.get_object("object", container="container")
             self.assertIsNone(res.data)
 
     def test_object_get_write_file(self):
         with requests_mock.Mocker() as m:
-            m.get("%scontainer/object" % self.endpoint, text="data")
+            m.get(f"{self.endpoint}container/object", text="data")
             with tempfile.NamedTemporaryFile() as f:
                 self.proxy.get_object(
                     "object", container="container", outfile=f.name
@@ -148,7 +146,7 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
 
     def test_object_get_remember_content(self):
         with requests_mock.Mocker() as m:
-            m.get("%scontainer/object" % self.endpoint, text="data")
+            m.get(f"{self.endpoint}container/object", text="data")
             res = self.proxy.get_object(
                 "object", container="container", remember_content=True
             )
@@ -271,7 +269,7 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
         segment_content = b''
         for index, (name, segment) in enumerate(segments.items()):
             self.assertEqual(
-                'test_container/test_image/{index:0>6}'.format(index=index),
+                f'test_container/test_image/{index:0>6}',
                 name,
             )
             segment_content += segment.read()
@@ -280,7 +278,7 @@ class TestObjectStoreProxy(test_proxy_base.TestProxyBase):
 
 class TestDownloadObject(base_test_object.BaseTestObject):
     def setUp(self):
-        super(TestDownloadObject, self).setUp()
+        super().setUp()
         self.the_data = b'test body'
         self.register_uris(
             [
@@ -353,7 +351,7 @@ class TestTempURL(TestObjectStoreProxy):
     expires_iso8601_format = '%Y-%m-%dT%H:%M:%SZ'
     short_expires_iso8601_format = '%Y-%m-%d'
     time_errmsg = (
-        'time must either be a whole number or in specific ' 'ISO 8601 format.'
+        'time must either be a whole number or in specific ISO 8601 format.'
     )
     path_errmsg = 'path must be full path to an object e.g. /v1/a/c/o'
     url = '/v1/AUTH_account/c/o'
@@ -361,7 +359,7 @@ class TestTempURL(TestObjectStoreProxy):
     key = 'correcthorsebatterystaple'
     method = 'GET'
     expected_url = url + (
-        '?temp_url_sig=temp_url_signature' '&temp_url_expires=1400003600'
+        '?temp_url_sig=temp_url_signature&temp_url_expires=1400003600'
     )
     expected_body = '\n'.join(
         [
@@ -602,7 +600,7 @@ class TestTempURL(TestObjectStoreProxy):
             )
         else:
             expected_url = self.expected_url.replace(
-                u'1400003600', u'2146636800'
+                '1400003600', '2146636800'
             )
         url = self.proxy.generate_temp_url(
             self.url,
@@ -654,38 +652,38 @@ class TestTempURL(TestObjectStoreProxy):
 
 
 class TestTempURLUnicodePathAndKey(TestTempURL):
-    url = u'/v1/\u00e4/c/\u00f3'
-    key = u'k\u00e9y'
+    url = '/v1/\u00e4/c/\u00f3'
+    key = 'k\u00e9y'
     expected_url = (
-        u'%s?temp_url_sig=temp_url_signature' u'&temp_url_expires=1400003600'
-    ) % url
-    expected_body = u'\n'.join(
+        f'{url}?temp_url_sig=temp_url_signature&temp_url_expires=1400003600'
+    )
+    expected_body = '\n'.join(
         [
-            u'GET',
-            u'1400003600',
+            'GET',
+            '1400003600',
             url,
         ]
     ).encode('utf-8')
 
 
 class TestTempURLUnicodePathBytesKey(TestTempURL):
-    url = u'/v1/\u00e4/c/\u00f3'
-    key = u'k\u00e9y'.encode('utf-8')
+    url = '/v1/\u00e4/c/\u00f3'
+    key = 'k\u00e9y'.encode()
     expected_url = (
-        u'%s?temp_url_sig=temp_url_signature' u'&temp_url_expires=1400003600'
-    ) % url
+        f'{url}?temp_url_sig=temp_url_signature&temp_url_expires=1400003600'
+    )
     expected_body = '\n'.join(
         [
-            u'GET',
-            u'1400003600',
+            'GET',
+            '1400003600',
             url,
         ]
     ).encode('utf-8')
 
 
 class TestTempURLBytesPathUnicodeKey(TestTempURL):
-    url = u'/v1/\u00e4/c/\u00f3'.encode('utf-8')
-    key = u'k\u00e9y'
+    url = '/v1/\u00e4/c/\u00f3'.encode()
+    key = 'k\u00e9y'
     expected_url = url + (
         b'?temp_url_sig=temp_url_signature' b'&temp_url_expires=1400003600'
     )
@@ -699,8 +697,8 @@ class TestTempURLBytesPathUnicodeKey(TestTempURL):
 
 
 class TestTempURLBytesPathAndKey(TestTempURL):
-    url = u'/v1/\u00e4/c/\u00f3'.encode('utf-8')
-    key = u'k\u00e9y'.encode('utf-8')
+    url = '/v1/\u00e4/c/\u00f3'.encode()
+    key = 'k\u00e9y'.encode()
     expected_url = url + (
         b'?temp_url_sig=temp_url_signature' b'&temp_url_expires=1400003600'
     )
@@ -714,7 +712,7 @@ class TestTempURLBytesPathAndKey(TestTempURL):
 
 
 class TestTempURLBytesPathAndNonUtf8Key(TestTempURL):
-    url = u'/v1/\u00e4/c/\u00f3'.encode('utf-8')
+    url = '/v1/\u00e4/c/\u00f3'.encode()
     key = b'k\xffy'
     expected_url = url + (
         b'?temp_url_sig=temp_url_signature' b'&temp_url_expires=1400003600'
